# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# service type constants:
L2GW = "L2GW"
IFACE_NAME_ATTR = 'interface_name'
NETWORK_ID = 'network_id'
SEG_ID = 'segmentation_id'
L2_SERVICE_RESOURCE_NAME = 'l2_gateway_service'
L2GATEWAY_ID = 'l2_gateway_id'
GATEWAY_RESOURCE_NAME = 'l2_gateway'
L2_GATEWAYS = 'l2-gateways'
DEVICE_ID_ATTR = 'device_name'
DEVICE_IP_ATTR = 'device_ip'
IFACE_NAME_ATTR = 'interfaces'
CONNECTION_RESOURCE_NAME = 'l2_gateway_connection'
EXT_ALIAS = 'l2-gateway-connection'
L2_GATEWAY_SERVICES = "l2-gateway-services"
L2_GATEWAYS_CONNECTION = "%ss" % EXT_ALIAS
L3_GATEWAY_NET = 'l3-gateway'
LOCAL = "local"



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""
PLUMgrid plugin agent extension unit tests
"""

import mock
from oslo_utils import importutils

from networking_plumgrid.neutron.plugins import plugin as plumgrid_plugin
from neutron.tests.unit.extensions import test_agent as agent

PLUM_DRIVER = ('networking_plumgrid.neutron.plugins.drivers.'
               'fake_plumlib.Plumlib')
FAKE_DIRECTOR = '1.1.1.1'
FAKE_PORT = '1234'
FAKE_USERNAME = 'fake_admin'
FAKE_PASSWORD = 'fake_password'
FAKE_TIMEOUT = '0'


class AgentTestCase(agent.AgentDBTestCase):
    _plugin_name = ('networking_plumgrid.neutron.plugins.'
                    'plugin.NeutronPluginPLUMgridV2')

    def setUp(self):
        def mocked_plumlib_init(self):
            director_plumgrid = FAKE_DIRECTOR
            director_port = FAKE_PORT
            director_username = FAKE_USERNAME
            director_password = FAKE_PASSWORD
            timeout = FAKE_TIMEOUT
            self._plumlib = importutils.import_object(PLUM_DRIVER)
            self._plumlib.director_conn(director_plumgrid,
                                        director_port, timeout,
                                        director_username,
                                        director_password)

        with mock.patch.object(plumgrid_plugin.NeutronPluginPLUMgridV2,
                               'plumgrid_init', new=mocked_plumlib_init):
            super(AgentTestCase, self).setUp()

    def tearDown(self):
        super(AgentTestCase, self).tearDown()



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension
from neutronclient.common import utils
from oslo_serialization import jsonutils


def _format_interfaces(physical_attachment_point):
    try:
        return '\n'.join([jsonutils.dumps(pap) for pap in
                          physical_attachment_point['interfaces']])
    except (TypeError, KeyError):
        return ''


class PhysicalAttachmentPoint(extension.NeutronClientExtension):
    resource = 'physical_attachment_point'
    resource_plural = 'physical_attachment_points'
    path = 'physical-attachment-points'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


def add_known_arguments(self, parser):
    parser.add_argument(
        '--interface',
        metavar='hostname=HOSTNAME,interface_name=INTERFACE-NAME',
        action='append', dest='interfaces', type=utils.str2dict,
        help=_('Hostnames and corresponding interfaces for physical'
               ' attachment point. '
               '(--interface option can be repeated)'))
    parser.add_argument('--hash_mode', dest='hash_mode', help=_('Hash mode'
                        ' for the physical attachment point. Default is L2'))
    parser.add_argument('--lacp', dest='lacp', help=_('LACP mode is enabled'
                        ' or disabled. Default is enabled. Options:'
                        ' True/False'))
    parser.add_argument('--transit_domain', dest='transit_domain_id',
                        help=_('Transit domain where'
                        ' physical attachment point should be'))


def args2body(self, parsed_args):
    try:
        if parsed_args.interfaces:
            interfaces = parsed_args.interfaces
        else:
            interfaces = []
        interface_dict = []
        for interface in interfaces:
            if "hostname" in interface and "interface_name" in interface:
                interface = {'hostname': interface['hostname'],
                             'interface': interface['interface_name']}
            else:
                raise KeyError("hostname and interface_name are both needed")
            interface_dict.append(interface)
        if parsed_args.name:
            pap_name = parsed_args.name
            body = {'physical_attachment_point': {'name': pap_name}}
        else:
            body = {'physical_attachment_point': {}}
        if parsed_args.interfaces:
            body['physical_attachment_point']['interfaces'] = interface_dict
        if parsed_args.lacp:
            if (str(parsed_args.lacp).lower() == 'false' or
                str(parsed_args.lacp).lower() == 'true'):
                body['physical_attachment_point']['lacp'] = parsed_args.lacp
            else:
                raise Exception("Please pass True, true, False or false"
                                " for LACP")
        if parsed_args.hash_mode:
            (body['physical_attachment_point']
                 ['hash_mode']) = parsed_args.hash_mode
        if parsed_args.transit_domain_id:
            (body['physical_attachment_point']
                 ['transit_domain_id']) = parsed_args.transit_domain_id
        return body
    except KeyError as err:
        raise Exception("KeyError: " + str(err))


class PhysicalAttachmentPointCreate(extension.ClientExtensionCreate,
                                    PhysicalAttachmentPoint):
    """Create a physical attachment point."""

    shell_command = 'physical-attachment-point-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='<PHYSICAL-ATTACHMENT-POINT-NAME>',
            help=_('Descriptive name for logical gateway.'))
        add_known_arguments(self, parser)

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            (body['physical_attachment_point']
                 ['tenant_id']) = parsed_args.tenant_id
        return body


class PhysicalAttachmentPointList(extension.ClientExtensionList,
                                  PhysicalAttachmentPoint):
    """List physical attachment points that belong to a given tenant."""

    shell_command = 'physical-attachment-point-list'
    list_columns = ['id', 'name', 'transit_domain_id', 'interfaces']
    pagination_support = True
    sorting_support = True


class PhysicalAttachmentPointShow(extension.ClientExtensionShow,
                                  PhysicalAttachmentPoint):
    """Show information of a given physical attachment point."""

    shell_command = 'physical-attachment-point-show'


class PhysicalAttachmentPointDelete(extension.ClientExtensionDelete,
                                    PhysicalAttachmentPoint):
    """Delete a given physical attachment point."""

    shell_command = 'physical-attachment-point-delete'


class PhysicalAttachmentPointUpdate(extension.ClientExtensionUpdate,
                                    PhysicalAttachmentPoint):
    """Update a given physical attachment point."""

    shell_command = 'physical-attachment-point-update'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name', metavar='name',
            help=_('Descriptive name for logical gateway.'))
        parser.add_argument(
            '--add_interface',
            metavar='hostname=HOSTNAME,interface_name=INTERFACE-NAME',
            action='append', dest='add_interfaces', type=utils.str2dict,
            help=_('Hostnames and corresponding interfaces for physical'
                   ' attachment point. '
                   '(--add_interface option can be repeated)'))
        parser.add_argument(
            '--remove_interface',
            metavar='hostname=HOSTNAME,interface_name=INTERFACE-NAME',
            action='append', dest='remove_interfaces', type=utils.str2dict,
            help=_('Hostnames and corresponding interfaces for physical'
                   ' attachment point. '
                   '(--remove_interface option can be repeated)'))
        parser.add_argument('--hash_mode', dest='hash_mode', help=_('Hash mode'
                            ' for the physical attachment point. Example would'
                            ' L2, L3 etc'))
        parser.add_argument('--lacp', dest='lacp', help=_('LACP mode can be'
                            ' enabled or disabled. Default is disabled.'
                            ' Options: True/False'))

    def args2body(self, parsed_args):
        try:
            if parsed_args.add_interfaces:
                add_interfaces = parsed_args.add_interfaces
            else:
                add_interfaces = []
            add_interface_dict = []
            for interface in add_interfaces:
                if "hostname" in interface and "interface_name" in interface:
                    interface = {'hostname': interface['hostname'],
                                 'interface': interface['interface_name']}
                else:
                    raise KeyError("hostname and interface_name are "
                                   "both needed")
                add_interface_dict.append(interface)
            if parsed_args.remove_interfaces:
                remove_interfaces = parsed_args.remove_interfaces
            else:
                remove_interfaces = []
            remove_interface_dict = []
            for interface in remove_interfaces:
                if "hostname" in interface and "interface_name" in interface:
                    interface = {'hostname': interface['hostname'],
                                 'interface': interface['interface_name']}
                else:
                    raise KeyError("hostname and interface_name "
                                   "are both needed")
                remove_interface_dict.append(interface)
            if parsed_args.name:
                pap_name = parsed_args.name
                body = {'physical_attachment_point': {'name': pap_name}}
            else:
                body = {'physical_attachment_point': {}}
            if parsed_args.add_interfaces:
                (body['physical_attachment_point'][
                 'add_interfaces']) = add_interface_dict
            if parsed_args.remove_interfaces:
                (body['physical_attachment_point']
                 ['remove_interfaces']) = remove_interface_dict
            if parsed_args.lacp:
                if (str(parsed_args.lacp).lower() == 'false' or
                    str(parsed_args.lacp).lower() == 'true'):
                    (body['physical_attachment_point']
                     ['lacp']) = parsed_args.lacp
                else:
                    raise Exception("Please pass True/true, False/false"
                                    " for LACP")
            if parsed_args.hash_mode:
                (body['physical_attachment_point']
                     ['hash_mode']) = parsed_args.hash_mode
            return body
        except KeyError as err:
            raise Exception("KeyError: " + str(err))



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from neutron.tests.api import base
from neutron.tests.tempest import test
from plumgrid_tempest_plugin import config
from plumgrid_tempest_plugin.tests.api import base_l2gw
import random
from tempest_lib.common.utils import data_utils

CONF = config.CONF


class L2GatewayExtensionTestJSON(base.BaseAdminNetworkTest):
    _interface = 'json'

    """
    Tests the following operations in the Neutron API using the REST client for
    Neutron:
        Create l2gateway
        List l2gateways
        Show l2gateway
        Update l2gateway
        Delete l2gateway
        Create l2gatewayconnection
        List l2gatewayconnections
        Show l2gatewayconnection
        Delete l2gatewayconnection
    """

    @classmethod
    def resource_setup(cls):
        super(L2GatewayExtensionTestJSON, cls).resource_setup()
        # Atleast one switch detail should be provided to run the tests
        if (len(CONF.network.l2gw_switch) < 0):
            msg = ('Atleast one switch detail must be defined.')
            raise cls.skipException(msg)
        if not test.is_extension_enabled('l2gateway', 'network'):
            msg = "L2Gateway Extension not enabled."
            raise cls.skipException(msg)

    @test.idempotent_id('3ca07946-a3c9-49ac-b058-8be54abecf1f')
    def test_create_show_list_update_delete_l2gateway(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gateway['id'])
        # Show details of L2Gateway
        show_body = self.admin_client.show_l2_gateway(l2_gateway['id'])
        self.assertEqual(gw_name, show_body['l2_gateway']['name'])
        conf_devices = base_l2gw.form_dict_devices(devices)
        create_devices = base_l2gw.form_dict_devices(show_body['l2_gateway']
                                                     ['devices'])
        # Validate the resource provided in the conf and created
        for k, v in zip(conf_devices.items(), create_devices.items()):
            self.assertEqual(k, v)
        # List L2Gateways
        self.admin_client.list_l2_gateways()
        # Update the name of an L2Gateway and verify the same
        updated_name = 'updated ' + gw_name
        update_body = self.admin_client.update_l2_gateway(l2_gateway['id'],
                                                          name=updated_name)
        self.assertEqual(update_body['l2_gateway']['name'], updated_name)
        show_body = self.admin_client.show_l2_gateway(l2_gateway['id'])
        self.assertEqual(show_body['l2_gateway']['name'], updated_name)

    @test.idempotent_id('3ad5e945-2b42-4ea8-9c03-0bf41d4167f2')
    def test_create_show_list_delete_l2gateway_connection(self):
        # Create a network
        name = data_utils.rand_name('network')
        net_body = self.admin_client.create_network(name=name)
        net_id = net_body['network']['id']
        self.addCleanup(self.admin_client.delete_network, net_id)
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        l2_gw_body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gw_id = l2_gw_body['l2_gateway']['id']
        seg_id = devices[0]['interfaces'][0]['segmentation_id'][0]
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gw_id)
        # Create an L2Gateway Connection
        l2_gw_conn_body = self.admin_client.create_l2_gateway_connection(
            l2_gateway_id=l2_gw_id, network_id=net_id, segmentation_id=seg_id)
        l2_gw_conn_id = l2_gw_conn_body['l2_gateway_connection']['id']
        self.addCleanup(self.admin_client.delete_l2_gateway_connection,
                        l2_gw_conn_id)
        # Show details of created L2 Gateway connection
        show_body = self.admin_client.show_l2_gateway_connection(l2_gw_conn_id)
        l2_gw_conn = show_body['l2_gateway_connection']
        self.assertEqual(net_id, l2_gw_conn['network_id'])
        self.assertEqual(l2_gw_id, l2_gw_conn['l2_gateway_id'])
        # List L2Gateway Connections
        self.admin_client.list_l2_gateway_connections()

    def test_create_l2gw_conn_with_segid_when_l2gw_created_with_segid(self):
        # Create an L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        devices = base_l2gw.get_l2gw_body(CONF.network.l2gw_switch)["devices"]
        if devices[0]['interfaces'][0]['segmentation_id']:
            seg_id = devices[0]['interfaces'][0]['segmentation_id'][0]
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip, devices=devices)
        l2_gateway = body['l2_gateway']
        l2_gw_id = l2_gateway['id']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gw_id)
        # Create a network
        name = data_utils.rand_name('network')
        net_body = self.admin_client.create_network(name=name)
        net_id = net_body['network']['id']
        self.addCleanup(self.admin_client.delete_network, net_id)
        # Create an L2Gateway Connection
        l2_gw_conn_body = self.admin_client.create_l2_gateway_connection(
            l2_gateway_id=l2_gw_id, network_id=net_id,
            segmentation_id=seg_id)
        l2_gw_conn_id = l2_gw_conn_body['l2_gateway_connection']['id']
        l2_gw_seg_id = l2_gw_conn_body['l2_gateway_connection'][
            'segmentation_id']
        self.addCleanup(self.admin_client.delete_l2_gateway_connection,
                        l2_gw_conn_id)
        # Show details of created L2 Gateway connection
        show_body = self.admin_client.show_l2_gateway_connection(
            l2_gw_conn_id)
        l2_gw_conn = show_body['l2_gateway_connection']
        self.assertEqual(net_id, l2_gw_conn['network_id'])
        self.assertEqual(l2_gw_id, l2_gw_conn['l2_gateway_id'])
        self.assertEqual(str(l2_gw_seg_id),
                         str(l2_gw_conn['segmentation_id']))

    def test_create_update_l2gw_with_multiple_devices(self):
        # Generating name for multi-device L2Gateway
        gw_name = data_utils.rand_name('l2gw')
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        # Generating a list consisting 3 random segmentation_ids
        seg_id = random.sample(range(2, 4095), 3)
        # Generating 3 device and interface names
        dev_name1 = data_utils.rand_name('device_name')
        interface_name1 = data_utils.rand_name('interface')
        dev_name2 = data_utils.rand_name('device_name')
        interface_name2 = data_utils.rand_name('interface')
        dev_name3 = data_utils.rand_name('device_name')
        interface_name3 = data_utils.rand_name('interface')
        device_name_list = [dev_name1, dev_name2, dev_name3]
        interface_name_list = [
            interface_name1, interface_name2, interface_name3]
        # Forming the device for multi-device L2Gateway
        devices_list = [{"device_name": device_name_list[0],
                         "device_ip": "192.168.10.100",
                         "interfaces":[{
                                 "segmentation_id": [str(seg_id[0])],
                                 "name": interface_name_list[0]}]}, {
                         "device_name": device_name_list[1],
                         "device_ip": "192.168.10.200",
                         "interfaces":[{
                                 "segmentation_id": [str(seg_id[1])],
                                 "name": interface_name_list[1]}]}, {
                         "device_name": device_name_list[2],
                         "device_ip": "192.168.10.50",
                         "interfaces":[{
                                 "segmentation_id": [str(seg_id[2])],
                                 "name": interface_name_list[2]}]}]
        # Create the multi-device L2gateway
        body = self.admin_client.create_l2_gateway(
            name=gw_name, vtep_ifc=vtep_ifc, vtep_ip=vtep_ip,
            devices=devices_list)
        l2_gateway = body['l2_gateway']
        l2_gw_id = l2_gateway['id']
        self.addCleanup(self.admin_client.delete_l2_gateway, l2_gw_id)
        # Check the created multi-device L2Gateway
        device_list = range(3)
        interface_list = range(3)
        show_body = self.admin_client.show_l2_gateway(l2_gw_id)
        self.assertEqual(gw_name, show_body['l2_gateway']['name'])
        self.assertEqual(l2_gateway['id'], show_body['l2_gateway']['id'])
        for i in range(3):
            device_list[i] = show_body['l2_gateway']['devices'][i][
                'device_name']
            interface_list[i] = show_body['l2_gateway']['devices'][i][
                'interfaces'][0]['name']
        for j in [0, 1, 2]:
            self.assertIn(device_name_list[j], device_list)
            self.assertIn(interface_name_list[j], interface_list)
        # Update the gateway device name
        device_list_updated = range(3)
        interface_list_updated = range(3)
        seg_id_updated = random.sample(range(2, 4095), 1)
        device_updated = [{"device_name": device_name_list[0],
                           "device_ip": "192.168.10.100",
                           "interfaces":[{"segmentation_id": seg_id_updated,
                                          "name": "intNameNew"}]}]
        interface_name_list = [
            interface_name_list[2], interface_name_list[1], "intNameNew"]
        body = self.admin_client.update_l2_gateway(
            l2_gateway['id'], devices=device_updated)
        show_body_updated = self.admin_client.show_l2_gateway(
            l2_gateway['id'])
        # Check updating of multi-device L2gateway
        self.assertEqual(gw_name, show_body_updated['l2_gateway']['name'])
        self.assertEqual(l2_gateway['id'],
                         show_body_updated['l2_gateway']['id'])
        for k in range(3):
            device_list_updated[k] = show_body_updated['l2_gateway'][
                'devices'][k]['device_name']
            interface_list_updated[k] = show_body_updated['l2_gateway'][
                'devices'][k]['interfaces'][0]['name']
        for l in range(3):
            self.assertIn(device_name_list[l], device_list_updated)
            self.assertIn(interface_name_list[l], interface_list_updated)



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron.db import model_base
from neutron.db import models_v2

import sqlalchemy as sa
from sqlalchemy import orm


class PGL2GatewayConnection(model_base.BASEV2, models_v2.HasTenant,
                          models_v2.HasId):
    """Define an l2 gateway connection between a l2 gateway and a network."""
    l2_gateway_id = sa.Column(sa.String(36),
                              sa.ForeignKey('pgl2gateways.id',
                                            ondelete='CASCADE'))
    network_id = sa.Column(sa.String(36),
                           sa.ForeignKey('networks.id', ondelete='CASCADE'))
    segmentation_id = sa.Column(sa.Integer)
    __table_args__ = (sa.UniqueConstraint(l2_gateway_id,
                                          network_id),)


class PGL2GatewayInterface(model_base.BASEV2, models_v2.HasId):
    """Define an l2 gateway interface."""
    interface_name = sa.Column(sa.String(255))
    device_id = sa.Column(sa.String(36),
                          sa.ForeignKey('pgl2gatewaydevices.id',
                                        ondelete='CASCADE'))
    segmentation_id = sa.Column(sa.Integer)


class PGL2GatewayDevice(model_base.BASEV2, models_v2.HasId):
    """Define an l2 gateway device."""
    device_name = sa.Column(sa.String(255))
    device_ip = sa.Column(sa.String(255))
    interfaces = orm.relationship(PGL2GatewayInterface,
                                  backref='pgl2gatewaydevices',
                                  cascade='all,delete')
    l2_gateway_id = sa.Column(sa.String(36),
                              sa.ForeignKey('pgl2gateways.id',
                                            ondelete='CASCADE'))


class PGL2Gateway(model_base.BASEV2, models_v2.HasId,
                models_v2.HasTenant):
    """Define an l2 gateway."""
    name = sa.Column(sa.String(255))
    vtep_ifc = sa.Column(sa.String(255))
    vtep_ip = sa.Column(sa.String(255))

    devices = orm.relationship(PGL2GatewayDevice,
                               backref='pgl2gateways',
                               cascade='all,delete')
    network_connections = orm.relationship(PGL2GatewayConnection,
                                           lazy='joined')



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


"""Neutron PLUMgrid Plugin exceptions"""

from networking_plumgrid._i18n import _
from neutron.common import exceptions as base_exec


class PLUMgridException(base_exec.NeutronException):
    message = _("PLUMgrid Plugin Error: %(err_msg)s")


class PLUMgridConnectionFailed(PLUMgridException):
    message = _("Connection failed with PLUMgrid Director: %(err_msg)s")


class TenantResourcesInUse(base_exec.NeutronException):
    message = _("TenantResourcesInUse: %(err_msg)s")



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

'''Implementation of SQLAlchemy backend.'''
import sys

from networking_plumgrid._i18n import _LW
from networking_plumgrid.neutron.plugins.common import exceptions as exception
from networking_plumgrid.neutron.plugins.db.sqlal import models
from oslo_config import cfg
from oslo_db.sqlalchemy import session as db_session
import sqlalchemy

from oslo_log import log as logging
LOG = logging.getLogger(__name__)

CONF = cfg.CONF

_facade = None


def get_facade():
    global _facade

    if not _facade:
        _facade = db_session.EngineFacade.from_config(CONF)
    return _facade

get_engine = lambda: get_facade().get_engine()
get_session = lambda: get_facade().get_session()


def create_table_pg_lock():
    try:
        meta = sqlalchemy.MetaData()
        meta.bind = get_engine()

        pg_lock = sqlalchemy.Table(
            'pg_lock', meta,
            sqlalchemy.Column('uuid', sqlalchemy.String(length=36),
                              primary_key=True,
                              nullable=False),
            sqlalchemy.Column('created_at', sqlalchemy.DateTime),
            sqlalchemy.Column('updated_at', sqlalchemy.DateTime),
            mysql_engine='InnoDB',
            mysql_charset='utf8'
        )
        pg_lock.create()
    except Exception:
        pass


def get_backend():
    """The backend is this module itself."""
    return sys.modules[__name__]


def model_query(context, *args):
    session = _session(context)
    query = session.query(*args)
    return query


def soft_delete_aware_query(context, *args, **kwargs):
    """Stack query helper that accounts for context's `show_deleted` field.

    :param show_deleted: if True, overrides context's show_deleted field.
    """

    query = model_query(context, *args)
    show_deleted = kwargs.get('show_deleted') or context.show_deleted

    if not show_deleted:
        query = query.filter_by(deleted_at=None)
    return query


def _session(context):
    return (context and context.session) or get_session()


def pg_lock_create(uuid):
    try:
        session = get_session()
        with session.begin():
            lock = session.query(models.PGLock).get(uuid)
            if lock is not None:
                return lock.uuid
            session.add(models.PGLock(uuid=uuid))
            LOG.debug("Lock acquired for resource: " + uuid)
    except:  # noqa
        LOG.warn(_LW("Lock contest, sending back to re-try: %s"), uuid)
        raise exception.TenantResourcesInUse


def pg_lock_get(uuid):
    try:
        import datetime
        current_time = datetime.datetime.utcnow()
        two_min_ago = current_time - datetime.timedelta(minutes=2)
        session = get_session()
        with session.begin():
            res = session.query(models.PGLock).filter(
                  models.PGLock.created_at < two_min_ago).all()
            return res
    except:  # noqa
        return None


def pg_lock_get_id(uuid):
    session = get_session()
    with session.begin():
        lock = session.query(models.PGLock).get(uuid)
        if lock is not None:
            return lock.uuid


def pg_lock_steal(uuid):
    session = get_session()
    with session.begin():
        rows_affected = session.query(
            models.PGLock
        ).filter_by(uuid=uuid).delete()
    if not rows_affected:
        return True


def pg_lock_release(uuid):
    session = get_session()
    with session.begin():
        rows_affected = session.query(
            models.PGLock
        ).filter_by(uuid=uuid).delete()
    if not rows_affected:
        return True



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""provider_net_binding

Revision ID: 2df53910135e
Revises: 51b361b5fa76
Create Date: 2015-12-03 21:39:21.991424

"""

# revision identifiers, used by Alembic.
revision = '2df53910135e'
down_revision = '51b361b5fa76'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'pg_provider_net_bindings',
        sa.Column('network_id', sa.String(length=36), nullable=False),
        sa.Column('network_type', sa.String(length=32), nullable=False),
        sa.Column('physical_network', sa.String(length=64), nullable=False),
        sa.Column('vlan_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ['network_id'], ['networks.id'], ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('network_id')
    )



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from networking_plumgrid._i18n import _
from neutron.common import exceptions
from neutron.db import models_v2
from neutron.plugins.ml2 import driver_api as api
from neutron.plugins.ml2 import models

import six
import sqlalchemy as sa
from sqlalchemy.orm import exc


class L2GatewayCommonDbMixin(object):

    def _apply_filters_to_query(self, query, model, filters):
        """Apply filters to query for the models."""
        if filters:
            for key, value in six.iteritems(filters):
                column = getattr(model, key, None)
                if column:
                    query = query.filter(column.in_(value))
        return query

    def _model_query(self, context, model):
        """Query model based on filter."""
        query = context.session.query(model)
        query_filter = None
        if not context.is_admin and hasattr(model, 'tenant_id'):
            if hasattr(model, 'shared'):
                query_filter = ((model.tenant_id == context.tenant_id) |
                                (model.shared == sa.true()))
            else:
                query_filter = (model.tenant_id == context.tenant_id)
        if query_filter is not None:
            query = query.filter(query_filter)
        return query

    def _get_collection_query(self, context, model, filters=None,
                              sorts=None, limit=None, marker_obj=None,
                              page_reverse=False):
        """Get collection query for the models."""
        collection = self._model_query(context, model)
        collection = self._apply_filters_to_query(collection, model, filters)
        return collection

    def _get_marker_obj(self, context, resource, limit, marker):
        """Get marker object for the resource."""
        if limit and marker:
            return getattr(self, '_get_%s' % resource)(context, marker)
        return None

    def _fields(self, resource, fields):
        """Get fields for the resource for get query."""
        if fields:
            return dict(((key, item) for key, item in resource.items()
                         if key in fields))
        return resource

    def _get_tenant_id_for_create(self, context, resource):
        """Get tenant id for creation of resources."""
        if context.is_admin and 'tenant_id' in resource:
            tenant_id = resource['tenant_id']
        elif ('tenant_id' in resource and
              resource['tenant_id'] != context.tenant_id):
            reason = _('Cannot create resource for another tenant')
            raise exceptions.AdminRequired(reason=reason)
        else:
            tenant_id = context.tenant_id
        return tenant_id

    def _get_collection(self, context, model, dict_func, filters=None,
                        fields=None, sorts=None, limit=None, marker_obj=None,
                        page_reverse=False):
        """Get collection object based on query for resources."""
        query = self._get_collection_query(context, model, filters=filters,
                                           sorts=sorts,
                                           limit=limit,
                                           marker_obj=marker_obj,
                                           page_reverse=page_reverse)
        items = [dict_func(c, fields) for c in query]
        if limit and page_reverse:
            items.reverse()
        return items

    def _get_network_segments(self, context, network_id):
        """Get network segments for the given network_id."""
        with context.session.begin(subtransactions=True):
            query = (context.session.query(models.NetworkSegment).
                     filter_by(network_id=network_id))
            records = query.all()
            return [self._make_segment_dict(record) for record in records]

    def _make_segment_dict(self, record):
        """Make a segment dictionary out of a DB record."""
        return {api.ID: record.id,
                api.NETWORK_TYPE: record.network_type,
                api.PHYSICAL_NETWORK: record.physical_network,
                api.SEGMENTATION_ID: record.segmentation_id}

    def _get_network(self, context, id):
        try:
            network = self._get_by_id(context, models_v2.Network, id)
        except exc.NoResultFound:
            raise exceptions.NetworkNotFound(net_id=id)
        return network

    def _get_by_id(self, context, model, id):
        query = self._model_query(context, model)
        return query.filter(model.id == id).one()



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
from sqlalchemy.dialects import mysql
from sqlalchemy.ext import mutable
from sqlalchemy import types


dumps = jsonutils.dumps
loads = jsonutils.loads


class LongText(types.TypeDecorator):
    impl = types.Text

    def load_dialect_impl(self, dialect):
        if dialect.name == 'mysql':
            return dialect.type_descriptor(mysql.LONGTEXT())
        else:
            return self.impl


class Json(LongText):

    def process_bind_param(self, value, dialect):
        return dumps(value)

    def process_result_value(self, value, dialect):
        if value is None:
            return None
        return loads(value)


class MutableList(mutable.Mutable, list):
    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, cls):
            if isinstance(value, list):
                return cls(value)
            return mutable.Mutable.coerce(key, value)
        else:
            return value

    def __delitem__(self, key):
        list.__delitem__(self, key)
        self.changed()

    def __setitem__(self, key, value):
        list.__setitem__(self, key, value)
        self.changed()

    def __getstate__(self):
        return list(self)

    def __setstate__(self, state):
        len = list.__len__(self)
        list.__delslice__(self, 0, len)
        list.__add__(self, state)
        self.changed()

    def append(self, value):
        list.append(self, value)
        self.changed()

    def remove(self, value):
        list.remove(self, value)
        self.changed()


class List(types.TypeDecorator):
    impl = types.Text

    def load_dialect_impl(self, dialect):
        if dialect.name == 'mysql':
            return dialect.type_descriptor(mysql.LONGTEXT())
        else:
            return self.impl

    def process_bind_param(self, value, dialect):
        return dumps(value)

    def process_result_value(self, value, dialect):
        if value is None:
            return None
        return loads(value)


MutableList.associate_with(List)
mutable.MutableDict.associate_with(LongText)
mutable.MutableDict.associate_with(Json)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
SQLAlchemy models for networking-plumgrid.
"""

from oslo_db.sqlalchemy import models
from oslo_utils import timeutils
import six
import sqlalchemy
from sqlalchemy.ext import declarative
from sqlalchemy.orm import session as orm_session

BASE = declarative.declarative_base()


def get_session():
    from networking_plumgrid.neutron.plugins.db.sqlal import api as db_api
    return db_api.get_session()


class PGBase(models.ModelBase, models.TimestampMixin):
    """Base class for networking-plumgrid"""
    __table_args__ = {'mysql_engine': 'InnoDB'}

    def expire(self, session=None, attrs=None):
        """Expire this object ()."""
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.expire(self, attrs)

    def refresh(self, session=None, attrs=None):
        """Refresh this object."""
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.refresh(self, attrs)

    def delete(self, session=None):
        """Delete this object."""
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.begin()
        session.delete(self)
        session.commit()

    def update_and_save(self, values, session=None):
        if not session:
            session = orm_session.Session.object_session(self)
            if not session:
                session = get_session()
        session.begin()
        for k, v in six.iteritems(values):
            setattr(self, k, v)
        session.commit()


class SoftDelete(object):
    deleted_at = sqlalchemy.Column(sqlalchemy.DateTime)

    def soft_delete(self, session=None):
        """Mark this object as deleted."""
        self.update_and_save({'deleted_at': timeutils.utcnow()},
                             session=session)


class StateAware(object):
    action = sqlalchemy.Column('action', sqlalchemy.String(255))
    status = sqlalchemy.Column('status', sqlalchemy.String(255))
    status_reason = sqlalchemy.Column('status_reason', sqlalchemy.Text)


class PGLock(BASE, PGBase):
    __tablename__ = 'pg_lock'

    uuid = sqlalchemy.Column(sqlalchemy.String(36),
                             primary_key=True)



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_plumgrid._i18n import _LI
from neutron.extensions import providernet as provider
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class Plumlib(object):
    """Class PLUMgrid Fake Library.

    This library is a by-pass implementation for the PLUMgrid Library.
    This class is being used by the unit test integration in Neutron.
    """

    def __init__(self):
        LOG.info(_LI('Python PLUMgrid Fake Library Started '))
        pass

    def director_conn(self, director_plumgrid, director_port, timeout,
                      director_admin, director_password):
        LOG.info(_LI('Fake Director: %s'),
                 director_plumgrid + ':' + str(director_port))
        pass

    def create_network(self, tenant_id, net_db, network, **kwargs):
        net_db["network"] = {}
        for key in (provider.NETWORK_TYPE,
                    provider.PHYSICAL_NETWORK,
                    provider.SEGMENTATION_ID):
            net_db["network"][key] = network["network"][key]
        return net_db

    def update_network(self, tenant_id, net_id, network, orig_net_db):
        pass

    def delete_network(self, net_db, net_id):
        pass

    def create_subnet(self, sub_db, net_db, ipnet):
        pass

    def update_subnet(self, orig_sub_db, new_sub_db, ipnet, net_db):
        pass

    def delete_subnet(self, tenant_id, net_db, net_id, sub_db):
        pass

    def create_port(self, port_db, router_db, subnet_db):
        pass

    def update_port(self, port_db, router_db, subnet_db):
        pass

    def delete_port(self, port_db, router_db):
        pass

    def create_router(self, tenant_id, router_db):
        pass

    def update_router(self, router_db, router_id):
        pass

    def delete_router(self, tenant_id, router_id):
        pass

    def add_router_interface(self, tenant_id, router_id, port_db, ipnet,
                             ip_version):
        pass

    def remove_router_interface(self, tenant_id, net_id, router_id):
        pass

    def create_floatingip(self, floating_ip):
        pass

    def update_floatingip(self, floating_ip_orig, floating_ip, id):
        pass

    def delete_floatingip(self, floating_ip_orig, id):
        pass

    def disassociate_floatingips(self, fip, port_id):
        return dict((key, fip[key]) for key in ("id", "floating_network_id",
                                                "floating_ip_address"))

    def create_security_group(self, sg_db):
        pass

    def update_security_group(self, sg_db):
        pass

    def delete_security_group(self, sg_db):
        pass

    def create_security_group_rule(self, sg_rule_db):
        pass

    def create_security_group_rule_bulk(self, sg_rule_db):
        pass

    def delete_security_group_rule(self, sg_rule_db):
        pass

    def create_l2_gateway(self, director_plumgrid,
                          director_admin,
                          director_password,
                          gateway_info,
                          vendor_type,
                          sw_username,
                          sw_password):
        pass

    def delete_l2_gateway(self, gw_info):
        pass

    def add_l2_gateway_connection(self, gw_conn_info):
        pass

    def delete_l2_gateway_connection(self, gw_conn_info):
        pass

    def create_physical_attachment_point(self, physical_attachment_point):
        pass

    def update_physical_attachment_point(self, physical_attachment_point):
        pass

    def delete_physical_attachment_point(self, pap_id):
        pass

    def create_transit_domain(self, transit_domain, db):
        pass

    def update_transit_domain(self, transit_domain, db):
        pass

    def delete_transit_domain(self, tvd_id):
        pass

    def get_available_interface(self):
        return "host1", "ifc1"



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from networking_plumgrid._i18n import _
from neutronclient.common import extension
from neutronclient.common import utils
from oslo_serialization import jsonutils

INTERFACE_DELIMITER = ";"
SEGMENTATION_ID_DELIMITER = "#"
INTERFACE_SEG_ID_DELIMITER = "|"


def _format_devices(l2_gateway):
    try:
        return '\n'.join([jsonutils.dumps(gateway) for gateway in
                          l2_gateway['devices']])
    except (TypeError, KeyError):
        return ''


class L2Gateway(extension.NeutronClientExtension):
    resource = 'l2_gateway'
    resource_plural = 'l2_gateways'
    path = 'l2-gateways'
    object_path = '/%s' % path
    resource_path = '/%s/%%s' % path
    versions = ['2.0']


def get_interface(interfaces):
    interface_dict = []
    for interface in interfaces:
        if INTERFACE_SEG_ID_DELIMITER in interface:
            int_name = interface.split(INTERFACE_SEG_ID_DELIMITER)[0]
            segid = interface.split(INTERFACE_SEG_ID_DELIMITER)[1]
            if SEGMENTATION_ID_DELIMITER in segid:
                segid = segid.split(SEGMENTATION_ID_DELIMITER)
            else:
                segid = [segid]
            interface_detail = {'name': int_name, 'segmentation_id': segid}
        else:
            interface_detail = {'name': interface}
        interface_dict.append(interface_detail)
    return interface_dict


def add_known_arguments(self, parser):
    parser.add_argument(
            '--vtep_ifc', dest='vtep_ifc',
            help=_('VTEP interface for logical gateway.'))
    parser.add_argument(
            '--vtep_ip', dest='vtep_ip',
            help=_('VTEP IP for logical gateway.'))
    parser.add_argument(
        '--device',
        metavar='name=name,device_ip=ip,interface_names=INTERFACE-DETAILS',
        action='append', dest='devices', type=utils.str2dict,
        help=_('Device name and Interface-names of l2gateway. '
               'Device ip of switch '
               'INTERFACE-DETAILS is of form '
               '\"<interface_name1>'
               '|<seg_id1>\" '
               '(--device option can be repeated)'))


def args2body(self, parsed_args):
        if parsed_args.devices:
            devices = parsed_args.devices
            interfaces = []
        else:
            devices = []
        device_dict = []
        for device in devices:
            if 'interface_names' in device.keys():
                interface = device['interface_names']
                if INTERFACE_DELIMITER in interface:
                    interface_dict = interface.split(INTERFACE_DELIMITER)
                    interfaces = get_interface(interface_dict)
                else:
                    interfaces = get_interface([interface])
            if 'name' in device.keys():
                dvc = {'device_name': device['name'],
                       'interfaces': interfaces}
            else:
                dvc = {'interfaces': interfaces}
            if 'device_ip' in device.keys():
                dvc.update({'device_ip': device['device_ip']})
            device_dict.append(dvc)
        if parsed_args.name and parsed_args.vtep_ifc and parsed_args.vtep_ip:
            l2gw_name = parsed_args.name
            vtep_ifc = parsed_args.vtep_ifc
            vtep_ip = parsed_args.vtep_ip
            body = {'l2_gateway': {'name': l2gw_name,
                                   'vtep_ifc': vtep_ifc,
                                   'vtep_ip': vtep_ip,
                                   'devices': device_dict}, }
        else:
            body = {'l2_gateway': {'devices': device_dict}, }
        return body


class L2GatewayCreate(extension.ClientExtensionCreate, L2Gateway):
    """Create l2gateway information."""

    shell_command = 'l2-gateway-create'

    def add_known_arguments(self, parser):
        parser.add_argument(
            'name', metavar='<GATEWAY-NAME>',
            help=_('Descriptive name for logical gateway.'))
        add_known_arguments(self, parser)

    def args2body(self, parsed_args):
        body = args2body(self, parsed_args)
        if parsed_args.tenant_id:
            body['l2_gateway']['tenant_id'] = parsed_args.tenant_id
        return body


class L2GatewayList(extension.ClientExtensionList, L2Gateway):
    """List l2gateway that belongs to a given tenant."""

    shell_command = 'l2-gateway-list'
    _formatters = {'devices': _format_devices, }
    list_columns = ['id', 'name', 'vtep_ifc', 'vtep_ip', 'devices']
    pagination_support = True
    sorting_support = True


class L2GatewayShow(extension.ClientExtensionShow, L2Gateway):
    """Show information of a given l2gateway."""

    shell_command = 'l2-gateway-show'


class L2GatewayDelete(extension.ClientExtensionDelete, L2Gateway):
    """Delete a given l2gateway."""

    shell_command = 'l2-gateway-delete'


class L2GatewayUpdate(extension.ClientExtensionUpdate, L2Gateway):
    """Update a given l2gateway."""

    shell_command = 'l2-gateway-update'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--name', metavar='name',
            help=_('Descriptive name for logical gateway.'))
        add_known_arguments(self, parser)

    def args2body(self, parsed_args):
        if parsed_args.devices:
            body = args2body(self, parsed_args)
        else:
            body = {'l2_gateway': {'name': parsed_args.name}}
        return body



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import logging as std_logging
import os

from oslo_config import cfg


def register_opt_group(conf, opt_group, options):
    conf.register_group(opt_group)
    for opt in options:
        conf.register_opt(opt, group=opt_group.name)


auth_group = cfg.OptGroup(name='auth',
                          title="Options for authentication and credentials")


AuthGroup = [
    cfg.StrOpt('test_accounts_file',
               default='etc/accounts.yaml',
               help="Path to the yaml file that contains the list of "
                    "credentials to use for running tests"),
    cfg.BoolOpt('allow_tenant_isolation',
                default=True,
                help="Allows test cases to create/destroy tenants and "
                     "users. This option requires that OpenStack Identity "
                     "API admin credentials are known. If false, isolated "
                     "test cases and parallel execution, can still be "
                     "achieved configuring a list of test accounts",
                deprecated_opts=[cfg.DeprecatedOpt('allow_tenant_isolation',
                                                   group='compute'),
                                 cfg.DeprecatedOpt('allow_tenant_isolation',
                                                   group='orchestration')]),
    cfg.BoolOpt('locking_credentials_provider',
                default=False,
                help="If set to True it enables the Accounts provider, "
                     "which locks credentials to allow for parallel execution "
                     "with pre-provisioned accounts. It can only be used to "
                     "run tests that ensure credentials cleanup happens. "
                     "It requires at least `2 * CONC` distinct accounts "
                     "configured in `test_accounts_file`, with CONC == the "
                     "number of concurrent test processes."),
    cfg.ListOpt('tempest_roles',
                help="Roles to assign to all users created by tempest",
                default=[])
]

identity_group = cfg.OptGroup(name='identity',
                              title="Keystone Configuration Options")

IdentityGroup = [
    cfg.StrOpt('catalog_type',
               default='identity',
               help="Catalog type of the Identity service."),
    cfg.BoolOpt('disable_ssl_certificate_validation',
                default=False,
                help="Set to True if using self-signed SSL certificates."),
    cfg.StrOpt('ca_certificates_file',
               default=None,
               help='Specify a CA bundle file to use in verifying a '
                    'TLS (https) server certificate.'),
    cfg.StrOpt('uri',
               help="Full URI of the OpenStack Identity API (Keystone), v2"),
    cfg.StrOpt('uri_v3',
               help='Full URI of the OpenStack Identity API (Keystone), v3'),
    cfg.StrOpt('auth_version',
               default='v2',
               help="Identity API version to be used for authentication "
                    "for API tests."),
    cfg.StrOpt('region',
               default='RegionOne',
               help="The identity region name to use. Also used as the other "
                    "services' region name unless they are set explicitly. "
                    "If no such region is found in the service catalog, the "
                    "first found one is used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the identity service."),
    cfg.StrOpt('username',
               help="Username to use for Nova API requests."),
    cfg.StrOpt('tenant_name',
               help="Tenant name to use for Nova API requests."),
    cfg.StrOpt('admin_role',
               default='admin',
               help="Role required to administrate keystone."),
    cfg.StrOpt('password',
               help="API key to use when authenticating.",
               secret=True),
    cfg.StrOpt('domain_name',
               help="Domain name for authentication (Keystone V3)."
                    "The same domain applies to user and project"),
    cfg.StrOpt('alt_username',
               help="Username of alternate user to use for Nova API "
                    "requests."),
    cfg.StrOpt('alt_tenant_name',
               help="Alternate user's Tenant name to use for Nova API "
                    "requests."),
    cfg.StrOpt('alt_password',
               help="API key to use when authenticating as alternate user.",
               secret=True),
    cfg.StrOpt('alt_domain_name',
               help="Alternate domain name for authentication (Keystone V3)."
                    "The same domain applies to user and project"),
    cfg.StrOpt('admin_username',
               help="Administrative Username to use for "
                    "Keystone API requests."),
    cfg.StrOpt('admin_tenant_name',
               help="Administrative Tenant name to use for Keystone API "
                    "requests."),
    cfg.StrOpt('admin_password',
               help="API key to use when authenticating as admin.",
               secret=True),
    cfg.StrOpt('admin_domain_name',
               help="Admin domain name for authentication (Keystone V3)."
                    "The same domain applies to user and project"),
]

identity_feature_group = cfg.OptGroup(name='identity-feature-enabled',
                                      title='Enabled Identity Features')

IdentityFeatureGroup = [
    cfg.BoolOpt('trust',
                default=True,
                help='Does the identity service have delegation and '
                     'impersonation enabled'),
    cfg.BoolOpt('api_v2',
                default=True,
                help='Is the v2 identity API enabled'),
    cfg.BoolOpt('api_v3',
                default=True,
                help='Is the v3 identity API enabled'),
]

compute_group = cfg.OptGroup(name='compute',
                             title='Compute Service Options')

ComputeGroup = [
    cfg.StrOpt('image_ref',
               help="Valid primary image reference to be used in tests. "
                    "This is a required option"),
    cfg.StrOpt('image_ref_alt',
               help="Valid secondary image reference to be used in tests. "
                    "This is a required option, but if only one image is "
                    "available duplicate the value of image_ref above"),
    cfg.StrOpt('flavor_ref',
               default="1",
               help="Valid primary flavor to use in tests."),
    cfg.StrOpt('flavor_ref_alt',
               default="2",
               help='Valid secondary flavor to be used in tests.'),
    cfg.StrOpt('image_ssh_user',
               default="root",
               help="User name used to authenticate to an instance."),
    cfg.StrOpt('image_ssh_password',
               default="password",
               help="Password used to authenticate to an instance."),
    cfg.StrOpt('image_alt_ssh_user',
               default="root",
               help="User name used to authenticate to an instance using "
                    "the alternate image."),
    cfg.StrOpt('image_alt_ssh_password',
               default="password",
               help="Password used to authenticate to an instance using "
                    "the alternate image."),
    cfg.IntOpt('build_interval',
               default=1,
               help="Time in seconds between build status checks."),
    cfg.IntOpt('build_timeout',
               default=300,
               help="Timeout in seconds to wait for an instance to build. "
                    "Other services that do not define build_timeout will "
                    "inherit this value."),
    cfg.BoolOpt('run_ssh',
                default=False,
                help="Should the tests ssh to instances?"),
    cfg.StrOpt('ssh_auth_method',
               default='keypair',
               help="Auth method used for authenticate to the instance. "
                    "Valid choices are: keypair, configured, adminpass. "
                    "keypair: start the servers with an ssh keypair. "
                    "configured: use the configured user and password. "
                    "adminpass: use the injected adminPass. "
                    "disabled: avoid using ssh when it is an option."),
    cfg.StrOpt('ssh_connect_method',
               default='fixed',
               help="How to connect to the instance? "
                    "fixed: using the first ip belongs the fixed network "
                    "floating: creating and using a floating ip"),
    cfg.StrOpt('ssh_user',
               default='root',
               help="User name used to authenticate to an instance."),
    cfg.IntOpt('ping_timeout',
               default=120,
               help="Timeout in seconds to wait for ping to "
                    "succeed."),
    cfg.IntOpt('ssh_timeout',
               default=300,
               help="Timeout in seconds to wait for authentication to "
                    "succeed."),
    cfg.IntOpt('ready_wait',
               default=0,
               help="Additional wait time for clean state, when there is "
                    "no OS-EXT-STS extension available"),
    cfg.IntOpt('ssh_channel_timeout',
               default=60,
               help="Timeout in seconds to wait for output from ssh "
                    "channel."),
    cfg.StrOpt('fixed_network_name',
               default='private',
               help="Name of the fixed network that is visible to all test "
                    "tenants."),
    cfg.StrOpt('network_for_ssh',
               default='public',
               help="Network used for SSH connections. Ignored if "
                    "use_floatingip_for_ssh=true or run_ssh=false."),
    cfg.IntOpt('ip_version_for_ssh',
               default=4,
               help="IP version used for SSH connections."),
    cfg.BoolOpt('use_floatingip_for_ssh',
                default=True,
                help="Does SSH use Floating IPs?"),
    cfg.StrOpt('catalog_type',
               default='compute',
               help="Catalog type of the Compute service."),
    cfg.StrOpt('region',
               default='',
               help="The compute region name to use. If empty, the value "
                    "of identity.region is used instead. If no such region "
                    "is found in the service catalog, the first found one is "
                    "used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the compute service."),
    cfg.StrOpt('path_to_private_key',
               help="Path to a private key file for SSH access to remote "
                    "hosts"),
    cfg.StrOpt('volume_device_name',
               default='vdb',
               help="Expected device name when a volume is attached to "
                    "an instance"),
    cfg.IntOpt('shelved_offload_time',
               default=0,
               help='Time in seconds before a shelved instance is eligible '
                    'for removing from a host.  -1 never offload, 0 offload '
                    'when shelved. This time should be the same as the time '
                    'of nova.conf, and some tests will run for as long as the '
                    'time.'),
    cfg.StrOpt('floating_ip_range',
               default='10.0.0.0/29',
               help='Unallocated floating IP range, which will be used to '
                    'test the floating IP bulk feature for CRUD operation. '
                    'This block must not overlap an existing floating IP '
                    'pool.')
]

compute_features_group = cfg.OptGroup(name='compute-feature-enabled',
                                      title="Enabled Compute Service Features")

ComputeFeaturesGroup = [
    cfg.BoolOpt('disk_config',
                default=True,
                help="If false, skip disk config tests"),
    cfg.ListOpt('api_extensions',
                default=['all'],
                help='A list of enabled compute extensions with a special '
                     'entry all which indicates every extension is enabled. '
                     'Each extension should be specified with alias name. '
                     'Empty list indicates all extensions are disabled'),
    cfg.BoolOpt('change_password',
                default=False,
                help="Does the test environment support changing the admin "
                     "password?"),
    cfg.BoolOpt('console_output',
                default=True,
                help="Does the test environment support obtaining instance "
                     "serial console output?"),
    cfg.BoolOpt('resize',
                default=False,
                help="Does the test environment support resizing?"),
    cfg.BoolOpt('pause',
                default=True,
                help="Does the test environment support pausing?"),
    cfg.BoolOpt('shelve',
                default=True,
                help="Does the test environment support shelving/unshelving?"),
    cfg.BoolOpt('suspend',
                default=True,
                help="Does the test environment support suspend/resume?"),
    cfg.BoolOpt('live_migration',
                default=True,
                help="Does the test environment support live migration "
                     "available?"),
    cfg.BoolOpt('block_migration_for_live_migration',
                default=False,
                help="Does the test environment use block devices for live "
                     "migration"),
    cfg.BoolOpt('block_migrate_cinder_iscsi',
                default=False,
                help="Does the test environment block migration support "
                     "cinder iSCSI volumes"),
    cfg.BoolOpt('vnc_console',
                default=False,
                help='Enable VNC console. This configuration value should '
                     'be same as [nova.vnc]->vnc_enabled in nova.conf'),
    cfg.BoolOpt('spice_console',
                default=False,
                help='Enable Spice console. This configuration value should '
                     'be same as [nova.spice]->enabled in nova.conf'),
    cfg.BoolOpt('rdp_console',
                default=False,
                help='Enable RDP console. This configuration value should '
                     'be same as [nova.rdp]->enabled in nova.conf'),
    cfg.BoolOpt('rescue',
                default=True,
                help='Does the test environment support instance rescue '
                     'mode?'),
    cfg.BoolOpt('enable_instance_password',
                default=True,
                help='Enables returning of the instance password by the '
                     'relevant server API calls such as create, rebuild '
                     'or rescue.'),
    cfg.BoolOpt('interface_attach',
                default=True,
                help='Does the test environment support dynamic network '
                     'interface attachment?'),
    cfg.BoolOpt('snapshot',
                default=True,
                help='Does the test environment support creating snapshot '
                     'images of running instances?'),
    cfg.BoolOpt('ec2_api',
                default=True,
                help='Does the test environment have the ec2 api running?')
]


image_group = cfg.OptGroup(name='image',
                           title="Image Service Options")

ImageGroup = [
    cfg.StrOpt('catalog_type',
               default='image',
               help='Catalog type of the Image service.'),
    cfg.StrOpt('region',
               default='',
               help="The image region name to use. If empty, the value "
                    "of identity.region is used instead. If no such region "
                    "is found in the service catalog, the first found one is "
                    "used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the image service."),
    cfg.StrOpt('http_image',
               default='http://download.cirros-cloud.net/0.3.1/'
               'cirros-0.3.1-x86_64-uec.tar.gz',
               help='http accessible image'),
    cfg.IntOpt('build_timeout',
               default=300,
               help="Timeout in seconds to wait for an image to "
                    "become available."),
    cfg.IntOpt('build_interval',
               default=1,
               help="Time in seconds between image operation status "
                    "checks.")
]

image_feature_group = cfg.OptGroup(name='image-feature-enabled',
                                   title='Enabled image service features')

ImageFeaturesGroup = [
    cfg.BoolOpt('api_v2',
                default=True,
                help="Is the v2 image API enabled"),
    cfg.BoolOpt('api_v1',
                default=True,
                help="Is the v1 image API enabled"),
]

network_group = cfg.OptGroup(name='network',
                             title='Network Service Options')

NetworkGroup = [
    cfg.StrOpt('l2gw_switch',
               default='',
               help='Switch name ,interface and vlan id information '),
    cfg.StrOpt('l2gw_switch_2',
               default='',
               help='Switch name ,interface and vlan id information'),
    cfg.StrOpt('hosts',
               default='',
               help='Network node and compute node host names and IPs'),
    cfg.StrOpt('l2gw_switch_ip',
               default='',
               help='Switch IP'),
    cfg.StrOpt('ovsdb_ip',
               default='',
               help='IP of ovsdb server'),
    cfg.IntOpt('ovsdb_port',
               default=6632,
               help='Port of ovsdb server'),
    cfg.StrOpt('ovsdb_schema_name',
               default='',
               help='schema name of ovsdb'),
    cfg.StrOpt('catalog_type',
               default='network',
               help='Catalog type of the Neutron service.'),
    cfg.StrOpt('region',
               default='',
               help="The network region name to use. If empty, the value "
                    "of identity.region is used instead. If no such region "
                    "is found in the service catalog, the first found one is "
                    "used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the network service."),
    cfg.StrOpt('tenant_network_cidr',
               default="10.100.0.0/16",
               help="The cidr block to allocate tenant ipv4 subnets from"),
    cfg.IntOpt('tenant_network_mask_bits',
               default=28,
               help="The mask bits for tenant ipv4 subnets"),
    cfg.StrOpt('tenant_network_v6_cidr',
               default="2003::/48",
               help="The cidr block to allocate tenant ipv6 subnets from"),
    cfg.IntOpt('tenant_network_v6_mask_bits',
               default=64,
               help="The mask bits for tenant ipv6 subnets"),
    cfg.BoolOpt('tenant_networks_reachable',
                default=False,
                help="Whether tenant network connectivity should be "
                     "evaluated directly"),
    cfg.StrOpt('public_network_id',
               default="",
               help="Id of the public network that provides external "
                    "connectivity"),
    cfg.StrOpt('public_router_id',
               default="",
               help="Id of the public router that provides external "
                    "connectivity. This should only be used when Neutron's "
                    "'allow_overlapping_ips' is set to 'False' in "
                    "neutron.conf. usually not needed past 'Grizzly' release"),
    cfg.IntOpt('build_timeout',
               default=300,
               help="Timeout in seconds to wait for network operation to "
                    "complete."),
    cfg.IntOpt('build_interval',
               default=1,
               help="Time in seconds between network operation status "
                    "checks."),
    cfg.ListOpt('dns_servers',
                default=["8.8.8.8", "8.8.4.4"],
                help="List of dns servers which should be used"
                     " for subnet creation"),
    cfg.StrOpt('port_vnic_type',
               choices=[None, 'normal', 'direct', 'macvtap'],
               help="vnic_type to use when Launching instances"
                    " with pre-configured ports."
                    " Supported ports are:"
                    " ['normal','direct','macvtap']"),
]

network_feature_group = cfg.OptGroup(name='network-feature-enabled',
                                     title='Enabled network service features')

NetworkFeaturesGroup = [
    cfg.BoolOpt('ipv6',
                default=True,
                help="Allow the execution of IPv6 tests"),
    cfg.ListOpt('api_extensions',
                default=['all'],
                help='A list of enabled network extensions with a special '
                     'entry all which indicates every extension is enabled. '
                     'Empty list indicates all extensions are disabled'),
    cfg.BoolOpt('ipv6_subnet_attributes',
                default=False,
                help="Allow the execution of IPv6 subnet tests that use "
                     "the extended IPv6 attributes ipv6_ra_mode "
                     "and ipv6_address_mode"
                ),
]

messaging_group = cfg.OptGroup(name='messaging',
                               title='Messaging Service')

MessagingGroup = [
    cfg.StrOpt('catalog_type',
               default='messaging',
               help='Catalog type of the Messaging service.'),
    cfg.IntOpt('max_queues_per_page',
               default=20,
               help='The maximum number of queue records per page when '
                    'listing queues'),
    cfg.IntOpt('max_queue_metadata',
               default=65536,
               help='The maximum metadata size for a queue'),
    cfg.IntOpt('max_messages_per_page',
               default=20,
               help='The maximum number of queue message per page when '
                    'listing (or) posting messages'),
    cfg.IntOpt('max_message_size',
               default=262144,
               help='The maximum size of a message body'),
    cfg.IntOpt('max_messages_per_claim',
               default=20,
               help='The maximum number of messages per claim'),
    cfg.IntOpt('max_message_ttl',
               default=1209600,
               help='The maximum ttl for a message'),
    cfg.IntOpt('max_claim_ttl',
               default=43200,
               help='The maximum ttl for a claim'),
    cfg.IntOpt('max_claim_grace',
               default=43200,
               help='The maximum grace period for a claim'),
]

volume_group = cfg.OptGroup(name='volume',
                            title='Block Storage Options')

VolumeGroup = [
    cfg.IntOpt('build_interval',
               default=1,
               help='Time in seconds between volume availability checks.'),
    cfg.IntOpt('build_timeout',
               default=300,
               help='Timeout in seconds to wait for a volume to become '
                    'available.'),
    cfg.StrOpt('catalog_type',
               default='volume',
               help="Catalog type of the Volume Service"),
    cfg.StrOpt('region',
               default='',
               help="The volume region name to use. If empty, the value "
                    "of identity.region is used instead. If no such region "
                    "is found in the service catalog, the first found one is "
                    "used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the volume service."),
    cfg.StrOpt('backend1_name',
               default='BACKEND_1',
               help="Name of the backend1 (must be declared in cinder.conf)"),
    cfg.StrOpt('backend2_name',
               default='BACKEND_2',
               help="Name of the backend2 (must be declared in cinder.conf)"),
    cfg.StrOpt('storage_protocol',
               default='iSCSI',
               help='Backend protocol to target when creating volume types'),
    cfg.StrOpt('vendor_name',
               default='Open Source',
               help='Backend vendor to target when creating volume types'),
    cfg.StrOpt('disk_format',
               default='raw',
               help='Disk format to use when copying a volume to image'),
    cfg.IntOpt('volume_size',
               default=1,
               help='Default size in GB for volumes created by volumes tests'),
]

volume_feature_group = cfg.OptGroup(name='volume-feature-enabled',
                                    title='Enabled Cinder Features')

VolumeFeaturesGroup = [
    cfg.BoolOpt('multi_backend',
                default=False,
                help="Runs Cinder multi-backend test (requires 2 backends)"),
    cfg.BoolOpt('backup',
                default=True,
                help='Runs Cinder volumes backup test'),
    cfg.BoolOpt('snapshot',
                default=True,
                help='Runs Cinder volume snapshot test'),
    cfg.ListOpt('api_extensions',
                default=['all'],
                help='A list of enabled volume extensions with a special '
                     'entry all which indicates every extension is enabled. '
                     'Empty list indicates all extensions are disabled'),
    cfg.BoolOpt('api_v1',
                default=True,
                help="Is the v1 volume API enabled"),
    cfg.BoolOpt('api_v2',
                default=True,
                help="Is the v2 volume API enabled"),
]


object_storage_group = cfg.OptGroup(name='object-storage',
                                    title='Object Storage Service Options')

ObjectStoreGroup = [
    cfg.StrOpt('catalog_type',
               default='object-store',
               help="Catalog type of the Object-Storage service."),
    cfg.StrOpt('region',
               default='',
               help="The object-storage region name to use. If empty, the "
                    "value of identity.region is used instead. If no such "
                    "region is found in the service catalog, the first found "
                    "one is used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the object-store service."),
    cfg.IntOpt('container_sync_timeout',
               default=600,
               help="Number of seconds to time on waiting for a container "
                    "to container synchronization complete."),
    cfg.IntOpt('container_sync_interval',
               default=5,
               help="Number of seconds to wait while looping to check the "
                    "status of a container to container synchronization"),
    cfg.StrOpt('operator_role',
               default='Member',
               help="Role to add to users created for swift tests to "
                    "enable creating containers"),
    cfg.StrOpt('reseller_admin_role',
               default='ResellerAdmin',
               help="User role that has reseller admin"),
    cfg.StrOpt('realm_name',
               default='realm1',
               help="Name of sync realm. A sync realm is a set of clusters "
                    "that have agreed to allow container syncing with each "
                    "other. Set the same realm name as Swift's "
                    "container-sync-realms.conf"),
    cfg.StrOpt('cluster_name',
               default='name1',
               help="One name of cluster which is set in the realm whose name "
                    "is set in 'realm_name' item in this file. Set the "
                    "same cluster name as Swift's container-sync-realms.conf"),
]

object_storage_feature_group = cfg.OptGroup(
    name='object-storage-feature-enabled',
    title='Enabled object-storage features')

ObjectStoreFeaturesGroup = [
    cfg.ListOpt('discoverable_apis',
                default=['all'],
                help="A list of the enabled optional discoverable apis. "
                     "A single entry, all, indicates that all of these "
                     "features are expected to be enabled"),
    cfg.BoolOpt('container_sync',
                default=True,
                help="Execute (old style) container-sync tests"),
    cfg.BoolOpt('object_versioning',
                default=True,
                help="Execute object-versioning tests"),
    cfg.BoolOpt('discoverability',
                default=True,
                help="Execute discoverability tests"),
]

database_group = cfg.OptGroup(name='database',
                              title='Database Service Options')

DatabaseGroup = [
    cfg.StrOpt('catalog_type',
               default='database',
               help="Catalog type of the Database service."),
    cfg.StrOpt('db_flavor_ref',
               default="1",
               help="Valid primary flavor to use in database tests."),
    cfg.StrOpt('db_current_version',
               default="v1.0",
               help="Current database version to use in database tests."),
]

orchestration_group = cfg.OptGroup(name='orchestration',
                                   title='Orchestration Service Options')

OrchestrationGroup = [
    cfg.StrOpt('catalog_type',
               default='orchestration',
               help="Catalog type of the Orchestration service."),
    cfg.StrOpt('region',
               default='',
               help="The orchestration region name to use. If empty, the "
                    "value of identity.region is used instead. If no such "
                    "region is found in the service catalog, the first found "
                    "one is used."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the orchestration service."),
    cfg.IntOpt('build_interval',
               default=1,
               help="Time in seconds between build status checks."),
    cfg.IntOpt('build_timeout',
               default=1200,
               help="Timeout in seconds to wait for a stack to build."),
    cfg.StrOpt('instance_type',
               default='m1.micro',
               help="Instance type for tests. Needs to be big enough for a "
                    "full OS plus the test workload"),
    cfg.StrOpt('keypair_name',
               help="Name of existing keypair to launch servers with."),
    cfg.IntOpt('max_template_size',
               default=524288,
               help="Value must match heat configuration of the same name."),
    cfg.IntOpt('max_resources_per_stack',
               default=1000,
               help="Value must match heat configuration of the same name."),
]


telemetry_group = cfg.OptGroup(name='telemetry',
                               title='Telemetry Service Options')

TelemetryGroup = [
    cfg.StrOpt('catalog_type',
               default='metering',
               help="Catalog type of the Telemetry service."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the telemetry service."),
    cfg.BoolOpt('too_slow_to_test',
                default=True,
                help="This variable is used as flag to enable "
                     "notification tests")
]


dashboard_group = cfg.OptGroup(name="dashboard",
                               title="Dashboard options")

DashboardGroup = [
    cfg.StrOpt('dashboard_url',
               default='http://localhost/',
               help="Where the dashboard can be found"),
    cfg.StrOpt('login_url',
               default='http://localhost/auth/login/',
               help="Login page for the dashboard"),
]


data_processing_group = cfg.OptGroup(name="data_processing",
                                     title="Data Processing options")

DataProcessingGroup = [
    cfg.StrOpt('catalog_type',
               default='data_processing',
               help="Catalog type of the data processing service."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the data processing "
                    "service."),
]


data_processing_feature_group = cfg.OptGroup(
    name="data_processing-feature-enabled",
    title="Enabled Data Processing features")

DataProcessingFeaturesGroup = [
    cfg.ListOpt('plugins',
                default=["vanilla", "hdp"],
                help="List of enabled data processing plugins")
]


boto_group = cfg.OptGroup(name='boto',
                          title='EC2/S3 options')
BotoGroup = [
    cfg.StrOpt('ec2_url',
               default="http://localhost:8773/services/Cloud",
               help="EC2 URL"),
    cfg.StrOpt('s3_url',
               default="http://localhost:8080",
               help="S3 URL"),
    cfg.StrOpt('aws_secret',
               help="AWS Secret Key",
               secret=True),
    cfg.StrOpt('aws_access',
               help="AWS Access Key"),
    cfg.StrOpt('aws_zone',
               default="nova",
               help="AWS Zone for EC2 tests"),
    cfg.StrOpt('s3_materials_path',
               default="/opt/stack/devstack/files/images/"
                       "s3-materials/cirros-0.3.0",
               help="S3 Materials Path"),
    cfg.StrOpt('ari_manifest',
               default="cirros-0.3.0-x86_64-initrd.manifest.xml",
               help="ARI Ramdisk Image manifest"),
    cfg.StrOpt('ami_manifest',
               default="cirros-0.3.0-x86_64-blank.img.manifest.xml",
               help="AMI Machine Image manifest"),
    cfg.StrOpt('aki_manifest',
               default="cirros-0.3.0-x86_64-vmlinuz.manifest.xml",
               help="AKI Kernel Image manifest"),
    cfg.StrOpt('instance_type',
               default="m1.tiny",
               help="Instance type"),
    cfg.IntOpt('http_socket_timeout',
               default=3,
               help="boto Http socket timeout"),
    cfg.IntOpt('num_retries',
               default=1,
               help="boto num_retries on error"),
    cfg.IntOpt('build_timeout',
               default=60,
               help="Status Change Timeout"),
    cfg.IntOpt('build_interval',
               default=1,
               help="Status Change Test Interval"),
]

stress_group = cfg.OptGroup(name='stress', title='Stress Test Options')

StressGroup = [
    cfg.StrOpt('nova_logdir',
               help='Directory containing log files on the compute nodes'),
    cfg.IntOpt('max_instances',
               default=16,
               help='Maximum number of instances to create during test.'),
    cfg.StrOpt('controller',
               help='Controller host.'),
    # new stress options
    cfg.StrOpt('target_controller',
               help='Controller host.'),
    cfg.StrOpt('target_ssh_user',
               help='ssh user.'),
    cfg.StrOpt('target_private_key_path',
               help='Path to private key.'),
    cfg.StrOpt('target_logfiles',
               help='regexp for list of log files.'),
    cfg.IntOpt('log_check_interval',
               default=60,
               help='time (in seconds) between log file error checks.'),
    cfg.IntOpt('default_thread_number_per_action',
               default=4,
               help='The number of threads created while stress test.'),
    cfg.BoolOpt('leave_dirty_stack',
                default=False,
                help='Prevent the cleaning (tearDownClass()) between'
                     ' each stress test run if an exception occurs'
                     ' during this run.'),
    cfg.BoolOpt('full_clean_stack',
                default=False,
                help='Allows a full cleaning process after a stress test.'
                     ' Caution : this cleanup will remove every objects of'
                     ' every tenant.')
]


scenario_group = cfg.OptGroup(name='scenario', title='Scenario Test Options')

ScenarioGroup = [
    cfg.StrOpt('img_dir',
               default='/opt/stack/new/devstack/files/images/'
               'cirros-0.3.1-x86_64-uec',
               help='Directory containing image files'),
    cfg.StrOpt('img_file', deprecated_name='qcow2_img_file',
               default='cirros-0.3.1-x86_64-disk.img',
               help='Image file name'),
    cfg.StrOpt('img_disk_format',
               default='qcow2',
               help='Image disk format'),
    cfg.StrOpt('img_container_format',
               default='bare',
               help='Image container format'),
    cfg.StrOpt('ami_img_file',
               default='cirros-0.3.1-x86_64-blank.img',
               help='AMI image file name'),
    cfg.StrOpt('ari_img_file',
               default='cirros-0.3.1-x86_64-initrd',
               help='ARI image file name'),
    cfg.StrOpt('aki_img_file',
               default='cirros-0.3.1-x86_64-vmlinuz',
               help='AKI image file name'),
    cfg.StrOpt('ssh_user',
               default='cirros',
               help='ssh username for the image file'),
    cfg.IntOpt(
        'large_ops_number',
        default=0,
        help="specifies how many resources to request at once. Used "
        "for large operations testing."),
    # TODO(yfried): add support for dhcpcd
    cfg.StrOpt('dhcp_client',
               default='udhcpc',
               choices=["udhcpc", "dhclient"],
               help='DHCP client used by images to renew DCHP lease. '
                    'If left empty, update operation will be skipped. '
                    'Supported clients: "udhcpc", "dhclient"')
]


service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

ServiceAvailableGroup = [
    cfg.BoolOpt('cinder',
                default=True,
                help="Whether or not cinder is expected to be available"),
    cfg.BoolOpt('neutron',
                default=False,
                help="Whether or not neutron is expected to be available"),
    cfg.BoolOpt('glance',
                default=True,
                help="Whether or not glance is expected to be available"),
    cfg.BoolOpt('swift',
                default=True,
                help="Whether or not swift is expected to be available"),
    cfg.BoolOpt('nova',
                default=True,
                help="Whether or not nova is expected to be available"),
    cfg.BoolOpt('heat',
                default=False,
                help="Whether or not Heat is expected to be available"),
    cfg.BoolOpt('ceilometer',
                default=True,
                help="Whether or not Ceilometer is expected to be available"),
    cfg.BoolOpt('horizon',
                default=True,
                help="Whether or not Horizon is expected to be available"),
    cfg.BoolOpt('sahara',
                default=False,
                help="Whether or not Sahara is expected to be available"),
    cfg.BoolOpt('ironic',
                default=False,
                help="Whether or not Ironic is expected to be available"),
    cfg.BoolOpt('trove',
                default=False,
                help="Whether or not Trove is expected to be available"),
    cfg.BoolOpt('zaqar',
                default=False,
                help="Whether or not Zaqar is expected to be available"),
]

debug_group = cfg.OptGroup(name="debug",
                           title="Debug System")

DebugGroup = [
    cfg.StrOpt('trace_requests',
               default='',
               help="""A regex to determine which requests should be traced.

This is a regex to match the caller for rest client requests to be able to
selectively trace calls out of specific classes and methods. It largely
exists for test development, and is not expected to be used in a real deploy
of tempest. This will be matched against the discovered ClassName:method
in the test environment.

Expected values for this field are:

 * ClassName:test_method_name - traces one test_method
 * ClassName:setUp(Class) - traces specific setup functions
 * ClassName:tearDown(Class) - traces specific teardown functions
 * ClassName:_run_cleanups - traces the cleanup functions

If nothing is specified, this feature is not enabled. To trace everything
specify .* as the regex.
""")
]

input_scenario_group = cfg.OptGroup(name="input-scenario",
                                    title="Filters and values for"
                                          " input scenarios")

InputScenarioGroup = [
    cfg.StrOpt('image_regex',
               default='^cirros-0.3.1-x86_64-uec$',
               help="Matching images become parameters for scenario tests"),
    cfg.StrOpt('flavor_regex',
               default='^m1.nano$',
               help="Matching flavors become parameters for scenario tests"),
    cfg.StrOpt('non_ssh_image_regex',
               default='^.*[Ww]in.*$',
               help="SSH verification in tests is skipped"
                    "for matching images"),
    cfg.StrOpt('ssh_user_regex',
               default="[[\"^.*[Cc]irros.*$\", \"root\"]]",
               help="List of user mapped to regex "
                    "to matching image names."),
]


baremetal_group = cfg.OptGroup(name='baremetal',
                               title='Baremetal provisioning service options',
                               help='When enabling baremetal tests, Nova '
                                    'must be configured to use the Ironic '
                                    'driver. The following paremeters for the '
                                    '[compute] section must be disabled: '
                                    'console_output, interface_attach, '
                                    'live_migration, pause, rescue, resize '
                                    'shelve, snapshot, and suspend')

BaremetalGroup = [
    cfg.StrOpt('catalog_type',
               default='baremetal',
               help="Catalog type of the baremetal provisioning service"),
    cfg.BoolOpt('driver_enabled',
                default=False,
                help="Whether the Ironic nova-compute driver is enabled"),
    cfg.StrOpt('driver',
               default='fake',
               help="Driver name which Ironic uses"),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the baremetal provisioning "
                    "service"),
    cfg.IntOpt('active_timeout',
               default=300,
               help="Timeout for Ironic node to completely provision"),
    cfg.IntOpt('association_timeout',
               default=30,
               help="Timeout for association of Nova instance and Ironic "
                    "node"),
    cfg.IntOpt('power_timeout',
               default=60,
               help="Timeout for Ironic power transitions."),
    cfg.IntOpt('unprovision_timeout',
               default=60,
               help="Timeout for unprovisioning an Ironic node.")
]

cli_group = cfg.OptGroup(name='cli', title="cli Configuration Options")

CLIGroup = [
    cfg.BoolOpt('enabled',
                default=True,
                help="enable cli tests"),
    cfg.StrOpt('cli_dir',
               default='/usr/local/bin',
               help="directory where python client binaries are located"),
    cfg.BoolOpt('has_manage',
                default=True,
                help=("Whether the tempest run location has access to the "
                      "*-manage commands. In a pure blackbox environment "
                      "it will not.")),
    cfg.IntOpt('timeout',
               default=15,
               help="Number of seconds to wait on a CLI timeout"),
]

negative_group = cfg.OptGroup(name='negative', title="Negative Test Options")

NegativeGroup = [
    cfg.StrOpt('test_generator',
               default='tempest.common.' +
               'generator.negative_generator.NegativeTestGenerator',
               help="Test generator class for all negative tests"),
]

_opts = [
    (auth_group, AuthGroup),
    (compute_group, ComputeGroup),
    (compute_features_group, ComputeFeaturesGroup),
    (identity_group, IdentityGroup),
    (identity_feature_group, IdentityFeatureGroup),
    (image_group, ImageGroup),
    (image_feature_group, ImageFeaturesGroup),
    (network_group, NetworkGroup),
    (network_feature_group, NetworkFeaturesGroup),
    (messaging_group, MessagingGroup),
    (volume_group, VolumeGroup),
    (volume_feature_group, VolumeFeaturesGroup),
    (object_storage_group, ObjectStoreGroup),
    (object_storage_feature_group, ObjectStoreFeaturesGroup),
    (database_group, DatabaseGroup),
    (orchestration_group, OrchestrationGroup),
    (telemetry_group, TelemetryGroup),
    (dashboard_group, DashboardGroup),
    (data_processing_group, DataProcessingGroup),
    (data_processing_feature_group, DataProcessingFeaturesGroup),
    (boto_group, BotoGroup),
    (stress_group, StressGroup),
    (scenario_group, ScenarioGroup),
    (service_available_group, ServiceAvailableGroup),
    (debug_group, DebugGroup),
    (baremetal_group, BaremetalGroup),
    (input_scenario_group, InputScenarioGroup),
    (cli_group, CLIGroup),
    (negative_group, NegativeGroup)
]


def register_opts():
    for g, o in _opts:
        register_opt_group(cfg.CONF, g, o)


def list_opts():
    """Return a list of oslo.config options available.

    The purpose of this is to allow tools like the Oslo sample config file
    generator to discover the options exposed to users.
    """
    return [(g.name, o) for g, o in _opts]


# this should never be called outside of this class
class TempestConfigPrivate(object):
    """Provides OpenStack configuration information."""

    DEFAULT_CONFIG_DIR = os.path.join(
        os.path.abspath(os.path.dirname(os.path.dirname(__file__))),
        "etc")

    DEFAULT_CONFIG_FILE = "tempest.conf"

    def __getattr__(self, attr):
        # Handles config options from the default group
        return getattr(cfg.CONF, attr)

    def _set_attrs(self):
        self.auth = cfg.CONF.auth
        self.compute = cfg.CONF.compute
        self.compute_feature_enabled = cfg.CONF['compute-feature-enabled']
        self.identity = cfg.CONF.identity
        self.identity_feature_enabled = cfg.CONF['identity-feature-enabled']
        self.image = cfg.CONF.image
        self.image_feature_enabled = cfg.CONF['image-feature-enabled']
        self.network = cfg.CONF.network
        self.network_feature_enabled = cfg.CONF['network-feature-enabled']
        self.volume = cfg.CONF.volume
        self.volume_feature_enabled = cfg.CONF['volume-feature-enabled']
        self.object_storage = cfg.CONF['object-storage']
        self.object_storage_feature_enabled = cfg.CONF[
            'object-storage-feature-enabled']
        self.database = cfg.CONF.database
        self.orchestration = cfg.CONF.orchestration
        self.messaging = cfg.CONF.messaging
        self.telemetry = cfg.CONF.telemetry
        self.dashboard = cfg.CONF.dashboard
        self.data_processing = cfg.CONF.data_processing
        self.data_processing_feature_enabled = cfg.CONF[
            'data_processing-feature-enabled']
        self.boto = cfg.CONF.boto
        self.stress = cfg.CONF.stress
        self.scenario = cfg.CONF.scenario
        self.service_available = cfg.CONF.service_available
        self.debug = cfg.CONF.debug
        self.baremetal = cfg.CONF.baremetal
        self.input_scenario = cfg.CONF['input-scenario']
        self.cli = cfg.CONF.cli
        self.negative = cfg.CONF.negative
        cfg.CONF.set_default('domain_name', self.identity.admin_domain_name,
                             group='identity')
        cfg.CONF.set_default('alt_domain_name',
                             self.identity.admin_domain_name,
                             group='identity')

    def __init__(self, parse_conf=True, config_path=None):
        """Initialize a configuration from a conf directory and conf file."""
        super(TempestConfigPrivate, self).__init__()
        config_files = []
        failsafe_path = "/etc/tempest/" + self.DEFAULT_CONFIG_FILE

        if config_path:
            path = config_path
        else:
            # Environment variables override defaults...
            conf_dir = os.environ.get('TEMPEST_CONFIG_DIR',
                                      self.DEFAULT_CONFIG_DIR)
            conf_file = os.environ.get('TEMPEST_CONFIG',
                                       self.DEFAULT_CONFIG_FILE)

            path = os.path.join(conf_dir, conf_file)

        if not os.path.isfile(path):
            path = failsafe_path

        # only parse the config file if we expect one to exist. This is needed
        # to remove an issue with the config file up to date checker.
        if parse_conf:
            config_files.append(path)
        if os.path.isfile(path):
            cfg.CONF([], project='tempest', default_config_files=config_files)
        else:
            cfg.CONF([], project='tempest')
        register_opts()
        self._set_attrs()


class TempestConfigProxy(object):
    _config = None
    _path = None

    _extra_log_defaults = [
        ('keystoneclient.session', std_logging.INFO),
        ('paramiko.transport', std_logging.INFO),
        ('requests.packages.urllib3.connectionpool', std_logging.WARN),
    ]

    def _fix_log_levels(self):
        """Tweak the oslo log defaults."""
        for name, level in self._extra_log_defaults:
            std_logging.getLogger(name).setLevel(level)

    def __getattr__(self, attr):
        if not self._config:
            self._fix_log_levels()
            self._config = TempestConfigPrivate(config_path=self._path)

        return getattr(self._config, attr)

    def set_config_path(self, path):
        self._path = path


CONF = TempestConfigProxy()



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock
from networking_plumgrid.neutron.tests.unit.cli import test_cli20
from networking_plumgrid.neutronclient.phyattachmentpoint import (
    physical_attachment_point as phyattp)
from neutronclient import shell
import sys


class CLITestV20ExtensionPhyAttPJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionPhyAttPJSON, self).setUp(plurals={'tags':
                                                                   'tag'})

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        contrib.return_value = [("physical_attachment_point", phyattp)]
        return contrib

    def test_ext_cmd_loaded(self):
        """Tests physical attachment point  commands loaded."""
        shell.NeutronShell('2.0')
        ext_cmd = {'physical-attachment-point-list':
                   phyattp.PhysicalAttachmentPointList,
                   'physical-attachment-point-create':
                   phyattp.PhysicalAttachmentPointCreate,
                   'physical-attachment-point-update':
                   phyattp.PhysicalAttachmentPointUpdate,
                   'physical-attachment-point-delete':
                   phyattp.PhysicalAttachmentPointDelete,
                   'physical-attachment-point-show':
                   phyattp.PhysicalAttachmentPointShow}
        self.assertDictContainsSubset(ext_cmd, shell.COMMANDS['2.0'])

    def _create_physical_attachment_point(self, args, name, lacp, hash_mode,
                                          interface):
        resource = 'physical_attachment_point'
        cmd = phyattp.PhysicalAttachmentPointCreate(test_cli20.MyApp(
                                                    sys.stdout), None)
        position_names = ['name', 'lacp', 'hash_mode', 'interfaces']
        position_values = [name, lacp, hash_mode, interface]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def _update_physical_attachment_point(self, args, values):
        resource = 'physical_attachment_point'
        cmd = phyattp.PhysicalAttachmentPointUpdate(test_cli20.MyApp(
                                                    sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_create_physical_attachment_point(self):
        """Test Create physical attachment point."""

        name = 'phyattpoint1'
        lacp = 'True'
        hash_mode = 'L3'
        args = [name, '--lacp', lacp, '--hash_mode', hash_mode,
                '--interface', 'hostname=u1,interface_name=i1']
        interface = [{"hostname": "u1", "interface": "i1"}]
        self._create_physical_attachment_point(args, name, lacp,
                                               hash_mode, interface)

    def test_create_physical_attachment_point_with_multiple_interfaces(self):
        """Test Create physical attachment point for multiple interfaces."""

        name = 'phyattpoint1'
        lacp = 'True'
        hash_mode = 'L3'
        args = [name, '--lacp', lacp, '--hash_mode', hash_mode, '--interface',
                'hostname=u1,interface_name=i1', '--interface',
                'hostname=u2,interface_name=i2']
        interface = [{"hostname": "u1", "interface": "i1"},
                     {"hostname": "u2", "interface": "i2"}]
        self._create_physical_attachment_point(args, name, lacp,
                                               hash_mode, interface)

    def test_create_physical_attachment_point_lacp_off(self):
        """Test Create physical attachment point."""

        name = 'phyattpoint1'
        lacp = 'False'
        hash_mode = 'L2'
        args = [name, '--lacp', lacp, '--hash_mode', hash_mode,
                '--interface', 'hostname=u1,interface_name=i1']
        interface = [{"hostname": "u1", "interface": "i1"}]
        self._create_physical_attachment_point(args, name, lacp,
                                               hash_mode, interface)

    def test_list_physical_attachment_points(self):
        """Test List physical attachment points."""

        resources = "physical_attachment_points"
        cmd = phyattp.PhysicalAttachmentPointList(test_cli20.MyApp(
                                                  sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_physical_attachment_points_pagination(self):
        #FIXME(fawadkhaliq)
        self.skipTest("Pagination does not work right now")
        #resources = 'physical_attachment_points'
        #cmd = phyattp.PhysicalAttachmentPointList(test_cli20.MyApp(
        #                                           sys.stdout), None)
        #self._test_list_resources_with_pagination(resources, cmd)

    def test_list_physical_attachment_points_sort(self):
        """list physical attachment points: --sort-key name
           --sort-key id --sort-key asc --sort-key desc
        """
        resources = 'physical_attachment_points'
        cmd = phyattp.PhysicalAttachmentPointList(test_cli20.MyApp(
                                                  sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_physical_attachment_points_limit(self):
        """list physical attachment points: -P."""
        resources = 'physical_attachment_points'
        cmd = phyattp.PhysicalAttachmentPointList(test_cli20.MyApp(
                                                  sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_delete_physical_attachment_point(self):
        """Test Delete physical attachment points."""

        resource = 'physical_attachment_point'
        cmd = phyattp.PhysicalAttachmentPointDelete(test_cli20.MyApp(
                                                    sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_show_physical_attachment_point(self):
        """
        Test Show physical attachment point: --fields id
        --fields name myid.
        """

        resource = 'physical_attachment_point'
        cmd = phyattp.PhysicalAttachmentPointShow(test_cli20.MyApp(
                                                  sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])

    def test_update_physical_attachment_point(self):
        """Test Update physical attachment point hash mode."""

        args = ['myid', '--lacp', 'False']
        values = {'lacp': 'False'}
        self._update_physical_attachment_point(args, values)

    def test_update_physical_attachment_point_name(self):
        """Test Update physical attachment point name."""

        args = ['myid', '--name', 'myname']
        values = {'name': 'myname'}
        self._update_physical_attachment_point(args, values)

    def test_update_physical_attachment_point_hash_mode(self):
        """Test Update physical attachment point hash mode."""

        args = ['myid', '--name', 'myname', '--hash_mode', 'L2']
        values = {'name': 'myname', 'hash_mode': 'L2'}
        self._update_physical_attachment_point(args, values)



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Test cases for  Neutron PLUMgrid Plug-in
"""

import mock
from oslo_utils import importutils

from networking_plumgrid.neutron.plugins.extensions import portbindings
from networking_plumgrid.neutron.plugins import plugin as plumgrid_plugin
from neutron import context
from neutron.extensions import providernet as provider
from neutron import manager
from neutron.tests.unit import _test_extension_portbindings as test_bindings
from neutron.tests.unit.db import test_db_base_plugin_v2 as test_plugin

PLUM_DRIVER = ('networking_plumgrid.neutron.plugins.drivers.'
               'fake_plumlib.Plumlib')
FAKE_DIRECTOR = '1.1.1.1'
FAKE_PORT = '1234'
FAKE_USERNAME = 'fake_admin'
FAKE_PASSWORD = 'fake_password'
FAKE_TIMEOUT = '0'


class PLUMgridPluginV2TestCase(test_plugin.NeutronDbPluginV2TestCase):
    _plugin_name = ('networking_plumgrid.neutron.plugins.'
                    'plugin.NeutronPluginPLUMgridV2')

    def setUp(self):
        def mocked_plumlib_init(self):
            director_plumgrid = FAKE_DIRECTOR
            director_port = FAKE_PORT
            director_username = FAKE_USERNAME
            director_password = FAKE_PASSWORD
            timeout = FAKE_TIMEOUT
            self._plumlib = importutils.import_object(PLUM_DRIVER)
            self._plumlib.director_conn(director_plumgrid,
                                        director_port, timeout,
                                        director_username,
                                        director_password)

        with mock.patch.object(plumgrid_plugin.NeutronPluginPLUMgridV2,
                               'plumgrid_init', new=mocked_plumlib_init):
            super(PLUMgridPluginV2TestCase, self).setUp(self._plugin_name)

    def tearDown(self):
        super(PLUMgridPluginV2TestCase, self).tearDown()


class TestPlumgridPluginNetworksV2(test_plugin.TestNetworksV2,
                                   PLUMgridPluginV2TestCase):
    pass


class TestPlumgridV2HTTPResponse(test_plugin.TestV2HTTPResponse,
                                 PLUMgridPluginV2TestCase):
    pass


class TestPlumgridPluginPortsV2(test_plugin.TestPortsV2,
                                PLUMgridPluginV2TestCase):

    def test_range_allocation(self):
        self.skipTest("Plugin does not support Neutron allocation process")

    def test_create_port_with_ipv6_dhcp_stateful_subnet_in_fixed_ips(self):
        self.skipTest("Plugin does not support DHCPv6 with IPv6")


class TestPlumgridPluginSubnetsV2(test_plugin.TestSubnetsV2,
                                  PLUMgridPluginV2TestCase):
    _unsupported = (
        'test_create_subnet_default_gw_conflict_allocation_pool_returns_409',
        'test_create_subnet_defaults', 'test_create_subnet_gw_values',
        'test_create_subnet_ipv6_gw_values',
        'test_update_subnet_gateway_in_allocation_pool_returns_409',
        'test_update_subnet_allocation_pools',
        'test_update_subnet_allocation_pools_invalid_pool_for_cidr')

    def setUp(self):
        if self._testMethodName in self._unsupported:
            self.skipTest("Plugin does not support Neutron allocation process")
        super(TestPlumgridPluginSubnetsV2, self).setUp()

    def test_create_subnets_bulk_emulated_plugin_failure(self):
        self.skipTest("Temporarily skipped; will be removed")

    def test_delete_network(self):
        self.skipTest("Temporarily skipped; will be removed")

    def test_subnet_admin_delete(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tenant_context = context.Context('', 'not_admin')

        network1 = self._fake_network('network1')
        network1_ret = plugin.create_network(tenant_context, network1)

        subnet1 = self._fake_subnet(network1_ret['id'])
        plugin.create_subnet(tenant_context, subnet1)
        net_db = plugin.get_network(admin_context, network1_ret['id'])

        self.assertEqual(network1_ret['tenant_id'], net_db["tenant_id"])

    def _fake_network(self, name):
        data = {'network': {'name': name,
                            'admin_state_up': False,
                            'shared': False,
                            'router:external': [],
                            'provider:network_type': None,
                            'provider:segmentation_id': None,
                            'provider:physical_network': None}}
        return data

    def _fake_subnet(self, net_id):
        allocation_pools = [{'start': '10.0.0.2',
                             'end': '10.0.0.254'}]
        return {'subnet': {'name': net_id,
                           'network_id': net_id,
                           'gateway_ip': '10.0.0.1',
                           'dns_nameservers': ['10.0.0.2'],
                           'host_routes': [],
                           'cidr': '10.0.0.0/24',
                           'allocation_pools': allocation_pools,
                           'enable_dhcp': True,
                           'ip_version': 4}}

    def test_subnet_update_enable_dhcp_no_ip_available_returns_409_ipv6(self):
        self.skipTest("Plugin does not support DHCPv6 with IPv6")

    def test_create_subnet_ipv6_pd_gw_values(self):
        self.skipTest("Plugin does not support Prefix Delegation with IPv6")


class TestPlumgridPluginPortBinding(PLUMgridPluginV2TestCase,
                                    test_bindings.PortBindingsTestCase):
    VIF_TYPE = portbindings.VIF_TYPE_IOVISOR
    HAS_PORT_FILTER = True

    def setUp(self):
        super(TestPlumgridPluginPortBinding, self).setUp()


class TestPlumgridNetworkAdminState(PLUMgridPluginV2TestCase):
    def test_network_admin_state(self):
        name = 'network_test'
        admin_status_up = False
        tenant_id = 'tenant_test'
        network = {'network': {'name': name,
                               'admin_state_up': admin_status_up,
                               'tenant_id': tenant_id}}
        plugin = manager.NeutronManager.get_plugin()
        self.assertEqual(plugin._network_admin_state(network), network)


class TestPlumgridAllocationPool(PLUMgridPluginV2TestCase):
    def test_allocate_pools_for_subnet(self):
        cidr = '10.0.0.0/24'
        gateway_ip = '10.0.0.254'
        subnet = {'gateway_ip': gateway_ip,
                  'cidr': cidr,
                  'ip_version': 4}
        allocation_pool = [{"start": '10.0.0.2',
                            "end": '10.0.0.253'}]
        context = None
        plugin = manager.NeutronManager.get_plugin()
        pool = plugin._allocate_pools_for_subnet(context, subnet)
        self.assertEqual(allocation_pool, pool)

    def test_conflict_dhcp_gw_ip(self):
        cidr = '10.0.0.0/24'
        gateway_ip = '10.0.0.1'
        subnet = {'gateway_ip': gateway_ip,
                  'cidr': cidr,
                  'ip_version': 4}
        allocation_pool = [{"start": '10.0.0.3',
                            "end": '10.0.0.254'}]
        context = None
        plugin = manager.NeutronManager.get_plugin()
        pool = plugin._allocate_pools_for_subnet(context, subnet)
        self.assertEqual(allocation_pool, pool)


class TestPlumgridProvidernet(PLUMgridPluginV2TestCase):

    def test_create_provider_network(self):
        tenant_id = 'admin'
        data = {'network': {'name': 'net1',
                            'admin_state_up': True,
                            'tenant_id': tenant_id,
                            provider.NETWORK_TYPE: 'vlan',
                            provider.SEGMENTATION_ID: 3333,
                            provider.PHYSICAL_NETWORK: 'phy3333'}}

        network_req = self.new_create_request('networks', data, self.fmt)
        net = self.deserialize(self.fmt, network_req.get_response(self.api))
        plumlib = importutils.import_object(PLUM_DRIVER)
        plumlib.create_network(tenant_id, net, data)
        self.assertEqual(net['network'][provider.NETWORK_TYPE], 'vlan')
        self.assertEqual(net['network'][provider.SEGMENTATION_ID], 3333)
        self.assertEqual(net['network'][provider.PHYSICAL_NETWORK], 'phy3333')

    def test_create_external_network_with_pap_name(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pap = {"physical_attachment_point": {
                   "tenant_id": "test_tenant",
                   "name": "test_name",
                   "hash_mode": "L2+L3",
                   "transit_domain_id": self._create_transit_domain(
                                        admin_context, plugin),
                   "lacp": True,
                   "interfaces": []}}

        pap_ret = plugin.create_physical_attachment_point(
                      admin_context, pap)
        data = {'network': {'name': 'net1',
                            'admin_state_up': True,
                            'tenant_id': 'test_tenant',
                            provider.NETWORK_TYPE: 'vlan',
                            provider.SEGMENTATION_ID: 3333,
                            provider.PHYSICAL_NETWORK: pap_ret['name'],
                            'shared': False,
                            'router:external': True}}

        net_db = plugin.create_network(admin_context, data)
        self.assertEqual(net_db['network'][provider.NETWORK_TYPE], 'vlan')
        self.assertEqual(net_db['network'][provider.SEGMENTATION_ID], 3333)
        self.assertEqual(
            net_db['network'][provider.PHYSICAL_NETWORK], pap_ret['id'])

    def test_create_provider_non_external_non_shared_network(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        pap = {"physical_attachment_point": {
                   "tenant_id": "test_tenant",
                   "name": "test_name",
                   "hash_mode": "L2+L3",
                   "transit_domain_id": self._create_transit_domain(
                                        admin_context, plugin),
                   "lacp": True,
                   "interfaces": []}}

        pap_ret = plugin.create_physical_attachment_point(
                      admin_context, pap)
        data = {'network': {'name': 'net1',
                            'admin_state_up': True,
                            'tenant_id': 'test_tenant',
                            provider.NETWORK_TYPE: 'vlan',
                            provider.SEGMENTATION_ID: 3333,
                            provider.PHYSICAL_NETWORK: pap_ret['id'],
                            'shared': False,
                            'router:external': False}}

        net_db = plugin.create_network(admin_context, data)
        self.assertEqual(net_db['network'][provider.NETWORK_TYPE], 'vlan')
        self.assertEqual(net_db['network'][provider.SEGMENTATION_ID], 3333)
        self.assertEqual(
            net_db['network'][provider.PHYSICAL_NETWORK], pap_ret['id'])

    def _create_transit_domain(self, admin_context, plugin):
        td = {"transit_domain": {
                  "tenant_id": "test_tenant",
                  "name": "td",
                  "implicit": False}}
        res = plugin.create_transit_domain(admin_context, td)
        return res["id"]


class TestDisassociateFloatingIP(PLUMgridPluginV2TestCase):

    def test_disassociate_floating_ip(self):
        port_id = "abcdefgh"
        tenant_id = "94eb42de4e331"
        fip_net_id = "b843d18245678"
        fip_addr = "10.0.3.44"
        fip_id = "e623679734051"
        fip = {"router_id": "94eb42de4e331",
               "tenant_id": tenant_id,
               "floating_network_id": fip_net_id,
               "fixed_ip_address": "192.168.8.2",
               "floating_ip_address": fip_addr,
               "port_id": port_id,
               "id": fip_id}
        plumlib = importutils.import_object(PLUM_DRIVER)
        fip_res = plumlib.disassociate_floatingips(fip, port_id)
        self.assertEqual(fip_res["id"], fip_id)
        self.assertEqual(fip_res["floating_ip_address"], fip_addr)
        self.assertEqual(fip_res["floating_network_id"], fip_net_id)



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron.api import extensions
from neutron.api.v2 import attributes
from neutron.api.v2 import base
from neutron import manager

from networking_plumgrid.neutron.plugins.common import constants
from networking_plumgrid.neutron.plugins.common import l2gw_validators


RESOURCE_ATTRIBUTE_MAP = {
    constants.L2_GATEWAYS: {
        'id': {'allow_post': False, 'allow_put': False,
               'is_visible': True},
        'vtep_ifc': {'allow_post': True, 'allow_put': True,
                     'validate': {'type:string': None},
                     'is_visible': True, 'default': ''},
        'vtep_ip': {'allow_post': True, 'allow_put': True,
                    'validate': {'type:string': None},
                    'is_visible': True, 'default': ''},
        'name': {'allow_post': True, 'allow_put': True,
                 'validate': {'type:string': None},
                 'is_visible': True, 'default': ''},
        'devices': {'allow_post': True, 'allow_put': True,
                    'validate': {'type:l2gwdevice_list': None},
                    'is_visible': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'validate': {'type:string': None},
                      'required_by_policy': True,
                      'is_visible': True}
    },
}

validator_func = l2gw_validators.validate_gwdevice_list
attributes.validators['type:l2gwdevice_list'] = validator_func


class L2gateway(extensions.ExtensionDescriptor):

    """API extension for Layer-2 Gateway support."""

    @classmethod
    def get_name(cls):
        return "L2 Gateway"

    @classmethod
    def get_alias(cls):
        return "l2-gateway"

    @classmethod
    def get_description(cls):
        return "Connects Neutron networks with external networks at layer 2."

    @classmethod
    def get_updated(cls):
        return "2015-01-01T00:00:00-00:00"

    @classmethod
    def get_resources(cls):
        # This method registers the URL and the dictionary  of
        # attributes on the neutron-server.

        exts = []
        plugin = manager.NeutronManager.get_plugin()
        resource_name = constants.GATEWAY_RESOURCE_NAME
        collection_name = resource_name.replace('_', '-') + "s"
        params = RESOURCE_ATTRIBUTE_MAP.get(constants.L2_GATEWAYS,
                                            dict())
        controller = base.create_resource(
            collection_name, resource_name, plugin, params)
        ex = extensions.ResourceExtension(collection_name,
                                          controller)
        exts.append(ex)
        return exts



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""
PGLock object
"""

#from oslo_versionedobjects import fields

from networking_plumgrid.neutron.plugins.db import api as db_api


class PGLock(object):
    #fields = {
    #    'uuid': fields.StringField(),
    #    'created_at': fields.DateTimeField(read_only=True),
    #}

    @classmethod
    def create(cls, uuid):
        return db_api.pg_lock_create(uuid)

    @classmethod
    def get(cls, uuid):
        return db_api.pg_lock_get(uuid)

    @classmethod
    def steal(cls, uuid):
        return db_api.pg_lock_steal(uuid)

    @classmethod
    def release(cls, uuid):
        return db_api.pg_lock_release(uuid)

    @classmethod
    def get_lock_id(cls, uuid):
        return db_api.pg_lock_get_lock_id(uuid)



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

import mock

from networking_plumgrid.neutron.tests.unit.cli import test_cli20
from networking_plumgrid.neutronclient.l2gw_client_ext import (
    _l2_gateway as l2_gateway)

from neutronclient import shell


class CLITestV20ExtensionL2GWJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionL2GWJSON, self).setUp(plurals={'tags': 'tag'})

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        contrib.return_value = [("_l2_gateway", l2_gateway)]
        return contrib

    def test_ext_cmd_loaded(self):
        """Tests l2gw  commands loaded."""
        shell.NeutronShell('2.0')
        ext_cmd = {'l2-gateway-list': l2_gateway.L2GatewayList,
                   'l2-gateway-create': l2_gateway.L2GatewayCreate,
                   'l2-gateway-update': l2_gateway.L2GatewayUpdate,
                   'l2-gateway-delete': l2_gateway.L2GatewayDelete,
                   'l2-gateway-show': l2_gateway.L2GatewayShow}
        self.assertDictContainsSubset(ext_cmd, shell.COMMANDS['2.0'])

    def _create_l2_gateway(self, args, name, vtep_ifc, vtep_ip, device):
        resource = 'l2_gateway'
        cmd = l2_gateway.L2GatewayCreate(test_cli20.MyApp(sys.stdout), None)
        position_names = ['name', 'vtep_ifc', 'vtep_ip', 'devices']
        position_values = [name, vtep_ifc, vtep_ip, device]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def _update_l2gateway(self, args, values):
        resource = 'l2_gateway'
        cmd = l2_gateway.L2GatewayUpdate(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_create_l2gateway(self):
        """Test Create l2gateway."""

        name = 'l2gateway1'
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        args = [name, '--vtep_ifc=1', '--vtep_ip=3.3.3.3/32', '--device',
                'name=d1,device_ip=192.168.10.100,interface_names=i1']
        device = [{'device_name': 'd1', 'device_ip': '192.168.10.100',
                   'interfaces': [{'name': 'i1'}]}]
        self._create_l2_gateway(args, name, vtep_ifc, vtep_ip, device)

    def test_create_l2gateway_with_multiple_devices(self):
        """Test Create l2gateway for multiple devices."""

        name = 'l2gateway1'
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        args = [name, '--vtep_ifc=1', '--vtep_ip=3.3.3.3/32', '--device',
                'name=dev1,device_ip=192.168.10.100,interface_names=int1',
                '--device',
                'name=dev2,device_ip=192.168.10.200,interface_names=int2']
        devices = [{'device_ip': '192.168.10.100', 'device_name': 'dev1',
                    'interfaces': [{'name': 'int1'}]},
                   {'device_ip': '192.168.10.200', 'device_name': 'dev2',
                    'interfaces': [{'name': 'int2'}]}]
        self._create_l2_gateway(args, name, vtep_ifc, vtep_ip, devices)

    def test_create_l2gateway_with_multiple_interfaces(self):
        """Test Create l2gateway with multiple interfaces."""

        name = 'l2gw-mul-interface'
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        args = [name, '--vtep_ifc=1', '--vtep_ip=3.3.3.3/32', '--device',
                'name=d1,device_ip=192.168.10.100,interface_names=int1;int2']
        interfaces = [{'name': 'int1'}, {'name': 'int2'}]
        device = [{'device_ip': '192.168.10.100', 'device_name': 'd1',
                   'interfaces': interfaces}]
        self._create_l2_gateway(args, name, vtep_ifc, vtep_ip, device)

    def test_create_l2gateway_with_segmenation_id(self):
        """Test Create l2gateway with segmentation-id."""

        name = 'l2gw-seg-id'
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        args = [name, '--vtep_ifc=1', '--vtep_ip=3.3.3.3/32', '--device',
                'name=d1,device_ip=192.168.10.100,interface_names=int1|100']
        interfaces = [{'name': 'int1', "segmentation_id": ["100"]}]
        device = [{'device_ip': '192.168.10.100', 'device_name': 'd1',
                   'interfaces': interfaces}]
        self._create_l2_gateway(args, name, vtep_ifc, vtep_ip, device)

    def test_create_l2gateway_with_mul_segmenation_id(self):
        """Test Create l2gateway with multiple segmentation-ids."""

        name = 'l2gw-mul-seg-id'
        vtep_ifc = '1'
        vtep_ip = '3.3.3.3/32'
        args = [name, '--vtep_ifc=1', '--vtep_ip=3.3.3.3/32', '--device',
               'name=d1,device_ip=192.168.10.100,interface_names=int1|100#200']
        interfaces = [{'name': 'int1', "segmentation_id": ["100", "200"]}]
        device = [{'device_ip': '192.168.10.100', 'device_name': 'd1',
                   'interfaces': interfaces}]
        self._create_l2_gateway(args, name, vtep_ifc, vtep_ip, device)

    def test_list_l2gateway(self):
        """Test List l2gateways."""

        resources = "l2_gateways"
        cmd = l2_gateway.L2GatewayList(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_delete_l2gateway(self):
        """Test Delete l2gateway."""

        resource = 'l2_gateway'
        cmd = l2_gateway.L2GatewayDelete(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_show_l2gateway(self):
        """Test Show l2gateway: --fields id --fields name myid."""

        resource = 'l2_gateway'
        cmd = l2_gateway.L2GatewayShow(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])

    def test_update_l2gateway(self):
        """Test Update l2gateway."""

        args = ['myid', '--name', 'myname', '--vtep_ifc', '1', '--vtep_ip',
                '3.3.3.3/32', '--device',
                'name=d1,device_ip=192.168.10.100,interface_names=i1']
        values = {'name': 'myname',
                  'vtep_ifc': '1',
                  'vtep_ip': '3.3.3.3/32',
                  'devices': [{'device_name': 'd1',
                               'device_ip': '192.168.10.100',
                               'interfaces': [{'name': 'i1'}]}]}
        self._update_l2gateway(args, values)

    def test_update_l2gateway_name(self):
        """Test Update l2gatewayi name."""

        args = ['myid', '--name', 'myname']
        values = {'name': 'myname'}
        self._update_l2gateway(args, values)

    def test_update_l2gateway_with_multiple_interfaces(self):
        """Test Update l2gateway with multiple interfaces."""

        args = ['myid', '--name', 'myname', '--vtep_ifc', '1', '--vtep_ip',
                '3.3.3.3/32', '--device',
                'name=d1,device_ip=192.168.10.100,interface_names=i1;i2']
        values = {'name': 'myname',
                  'vtep_ifc': '1',
                  'vtep_ip': '3.3.3.3/32',
                  'devices': [{'device_name': 'd1',
                               'device_ip': '192.168.10.100',
                               'interfaces': [{'name': 'i1'},
                                              {'name': 'i2'}]}]}
        self._update_l2gateway(args, values)

    def test_update_l2gateway_with_segmentation_id(self):
        """Test Update l2gateway with segmentation-id."""

        args = ['myid', '--name', 'myname', '--vtep_ifc', '1', '--vtep_ip',
                '3.3.3.3/32', '--device',
                'name=d1,device_ip=192.168.10.100,interface_names=int1|100']
        interfaces = [{'name': 'int1', "segmentation_id": ["100"]}]
        values = {'name': 'myname',
                  'vtep_ifc': '1',
                  'vtep_ip': '3.3.3.3/32',
                  'devices': [{'device_name': 'd1',
                               'device_ip': '192.168.10.100',
                               'interfaces': interfaces}]}
        self._update_l2gateway(args, values)



# Copyright 2016 PLUMgrid, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# service type constants:


def _retrieve_subnet_dict(self, port_db, context):
    """
    Helper function to retrieve the subnet dictionary
    """
    # Retrieve subnet information
    subnet_db = {}
    if len(port_db["fixed_ips"]) != 0:
        for subnet_index in range(0,
                                  len(port_db["fixed_ips"])):
            subnet_id = (port_db["fixed_ips"][subnet_index]
                         ["subnet_id"])
            subnet_db[subnet_index] = self._get_subnet(context, subnet_id)
    return subnet_db



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""phy_att_point

Revision ID: 351c4f5710e7
Revises: 1be40493866d
Create Date: 2015-12-14 07:01:48.954910

"""

# revision identifiers, used by Alembic.
revision = '351c4f5710e7'
down_revision = '1be40493866d'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'pg_transit_domains',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('tenant_id', sa.String(length=36), nullable=True),
        sa.Column('implicit', sa.Boolean(), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table(
        'pg_physical_attachment_points',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('tenant_id', sa.String(length=36), nullable=True),
        sa.Column('implicit', sa.Boolean(), nullable=True),
        sa.Column('lacp', sa.Boolean(), nullable=False),
        sa.Column('hash_mode', sa.String(length=255), nullable=True),
        sa.Column('transit_domain_id', sa.String(length=36), nullable=True),
        sa.ForeignKeyConstraint(
            ['transit_domain_id'], ['pg_transit_domains.id'],
            ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )

    op.create_table(
        'pg_physical_attachment_point_interfaces',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('hostname', sa.String(length=255), nullable=False),
        sa.Column('interface', sa.String(length=255), nullable=True),
        sa.Column('pap_id', sa.String(length=36), nullable=True),
        sa.ForeignKeyConstraint(
            ['pap_id'], ['pg_physical_attachment_points.id'],
            ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock
from networking_plumgrid.neutron.tests.unit.cli import test_cli20
from networking_plumgrid.neutronclient.transitdomain import (
    transitdomain as transitdom)
from neutronclient import shell
import sys


class CLITestV20ExtensionTransitDomainJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        # need to mock before super because extensions loaded on instantiation
        self._mock_extension_loading()
        super(CLITestV20ExtensionTransitDomainJSON,
              self).setUp(plurals={'tags': 'tag'})

    def _mock_extension_loading(self):
        ext_pkg = 'neutronclient.common.extension'
        contrib = mock.patch(ext_pkg + '._discover_via_entry_points').start()
        contrib.return_value = [("transit_domain", transitdom)]
        return contrib

    def test_ext_cmd_loaded(self):
        """Tests transit domain commands loaded."""
        shell.NeutronShell('2.0')
        ext_cmd = {'transit-domain-list':
                   transitdom.TransitDomainList,
                   'transit-domain-create':
                   transitdom.TransitDomainCreate,
                   'transit-domain-update':
                   transitdom.TransitDomainUpdate,
                   'transit-domain-delete':
                   transitdom.TransitDomainDelete,
                   'transit-domain-show':
                   transitdom.TransitDomainShow}
        self.assertDictContainsSubset(ext_cmd, shell.COMMANDS['2.0'])

    def _create_transit_domain(self, args, name):
        resource = 'transit_domain'
        cmd = transitdom.TransitDomainCreate(test_cli20.MyApp(
                                             sys.stdout), None)
        position_names = ['name']
        position_values = [name]
        self._test_create_resource(resource, cmd, name, 'myid', args,
                                   position_names, position_values)

    def _update_transit_domain(self, args, values):
        resource = 'transit_domain'
        cmd = transitdom.TransitDomainUpdate(test_cli20.MyApp(
                                             sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   args, values)

    def test_create_transit_domain(self):
        """Test create transit domain"""

        name = 'transitdomain1'
        args = [name]
        self._create_transit_domain(args, name)

    def test_list_transit_domains(self):
        """Test List transit domains."""

        resources = "transit_domains"
        cmd = transitdom.TransitDomainList(test_cli20.MyApp(
                                           sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_transit_domains_sort(self):
        """list transit domains: --sort-key name
           --sort-key id --sort-key asc --sort-key desc
        """
        resources = "transit_domains"
        cmd = transitdom.TransitDomainList(test_cli20.MyApp(
                                           sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_transit_domains_limit(self):
        """list transit domains: -P."""
        resources = "transit_domains"
        cmd = transitdom.TransitDomainList(test_cli20.MyApp(
                                           sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_delete_transit_domain_point(self):
        """Test delete transit domain"""
        resource = "transit_domain"
        cmd = transitdom.TransitDomainDelete(test_cli20.MyApp(
                                             sys.stdout), None)

        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_show_transit_domain_point(self):
        """
        Test show transit domain: --fields id
        --fields name myid.
        """

        resource = 'transit_domain'
        cmd = transitdom.TransitDomainShow(test_cli20.MyApp(
                                           sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])



# Copyright 2015 PLUMgrid, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Transit domain extension unit tests
"""

from networking_plumgrid.neutron.plugins.extensions import \
    transitdomain as ext_tvd
from networking_plumgrid.neutron.tests.unit import \
    test_networking_plumgrid as test_pg

from neutron.api import extensions
from neutron import context
from neutron import manager
from neutron.tests.unit.api import test_extensions as test_ext
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class TransitDomainExtensionManager(object):

    def get_resources(self):
        return ext_tvd.Transitdomain.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class TransitDomainTestCase(test_pg.PLUMgridPluginV2TestCase):
    def setUp(self, plugin=None, ext_mgr=None):
        super(TransitDomainTestCase, self).setUp()
        ext_mgr = TransitDomainExtensionManager()
        extensions.PluginAwareExtensionManager._instance = None
        self.ext_api = test_ext.setup_extensions_middleware(ext_mgr)


class TestTransitDomain(TransitDomainTestCase):
    def test_create_transit_domain(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()

        tvd = self._make_tvd_dict()

        tvd_ret = plugin.create_transit_domain(
                      admin_context, tvd)
        tvd["transit_domain"]["id"] = tvd_ret["id"]
        self.assertEqual(tvd_ret, tvd["transit_domain"])

    def test_update_transit_domain(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tvd = self._make_tvd_dict()
        tvd_old_ret = plugin.create_transit_domain(admin_context, tvd)
        tvd_new = self._make_tvd_dict("updated_name")
        tvd_ret = plugin.update_transit_domain(admin_context,
                  tvd_old_ret["id"], tvd_new)
        tvd_new["transit_domain"]["id"] = tvd_old_ret["id"]
        self.assertEqual(tvd_ret, tvd_new["transit_domain"])

    def test_get_transit_domain(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tvd = self._make_tvd_dict()
        tvd_create = plugin.create_transit_domain(
                      admin_context, tvd)
        tvd_get = plugin.get_transit_domain(
                      admin_context, tvd_create["id"])
        self.assertEqual(tvd_create, tvd_get)

    def test_delete_transit_domain(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tvd = self._make_tvd_dict()
        tvd_create = plugin.create_transit_domain(
                      admin_context, tvd)
        result = plugin.delete_transit_domain(admin_context,
                                  tvd_create["id"])
        self.assertIsNone(result)

    def test_list_transit_domain(self):
        plugin = manager.NeutronManager.get_plugin()
        admin_context = context.get_admin_context()
        tvd1 = self._make_tvd_dict("t1")
        tvd1_ret = plugin.create_transit_domain(
                      admin_context, tvd1)
        tvd2 = self._make_tvd_dict("t2")
        tvd2_ret = plugin.create_transit_domain(
                      admin_context, tvd2)
        items = []
        items.append(tvd1_ret)
        items.append(tvd2_ret)
        res = plugin.get_transit_domains(admin_context)
        self.assertItemsEqual([i['id'] for i in res],
                              [i['id'] for i in items])

    def _make_tvd_dict(self, name="test"):
        return {"transit_domain": {
                   "tenant_id": "test_tenant",
                   "name": name,
                   "implicit": False}}
